# -*- coding: utf-8 -*-
"""
本模块功能：证券投资组合理论计算函数包
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2020年7月1日
最新修订日期：2020年7月29日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
作者邮件：wdehong2000@163.com
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""
#==============================================================================
#统一屏蔽一般性警告
import warnings; warnings.filterwarnings("ignore")     
from siat.common import *
from siat.translate import *
from siat.security_prices import *
#==============================================================================
import matplotlib.pyplot as plt

#处理绘图汉字乱码问题
import sys; czxt=sys.platform
if czxt in ['win32','win64']:
    plt.rcParams['font.sans-serif'] = ['SimHei']  # 设置默认字体
    mpfrc={'font.family': 'SimHei'}

if czxt in ['darwin']: #MacOSX
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family': 'Heiti TC'}

if czxt in ['linux']: #website Jupyter
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family':'Heiti TC'}

# 解决保存图像时'-'显示为方块的问题
plt.rcParams['axes.unicode_minus'] = False 
#==============================================================================
#==============================================================================
def portfolio_config(tickerlist,sharelist):
    """
    将股票列表tickerlist和份额列表sharelist合成为一个字典
    """
    #整理sharelist的小数点
    ratiolist=[]
    for s in sharelist:
        ss=round(s,4); ratiolist=ratiolist+[ss]
    #合成字典
    new_dict=dict(zip(tickerlist,ratiolist))
    return new_dict

#==============================================================================
def ratiolist_round(sharelist,num=4):
    """
    将股票份额列表sharelist中的数值四舍五入
    """
    #整理sharelist的小数点
    ratiolist=[]
    for s in sharelist:
        ss=round(s,num); ratiolist=ratiolist+[ss]
    return ratiolist

#==============================================================================
def varname(p):
    """
    功能：获得变量的名字本身。
    """
    import inspect
    import re    
    for line in inspect.getframeinfo(inspect.currentframe().f_back)[3]:
        m = re.search(r'\bvarname\s*\(\s*([A-Za-z_][A-Za-z0-9_]*)\s*\)', line)
        if m:
            return m.group(1)    

#==============================================================================
def get_start_date(end_date,pastyears=1):
    """
    输入参数：一个日期，年数
    输出参数：几年前的日期
    start_date, end_date是datetime类型
    """
    import pandas as pd
    try:
        end_date=pd.to_datetime(end_date)
    except:
        print("#Error(): invalid date,",end_date)
        return None
    
    from datetime import datetime,timedelta
    start_date=datetime(end_date.year-pastyears,end_date.month,end_date.day)
    start_date=start_date-timedelta(days=1)
    # 日期-1是为了保证计算收益率时得到足够的样本数量
    return start_date

#==============================================================================
if __name__=='__main__':
    pass

def cumulative_returns_plot(retgroup,name_list,titletxt,ylabeltxt,xlabeltxt):
    # 累积收益曲线绘制函数
    
    lslist=['-','--',':','-.']
    for name in name_list:
        pos=name_list.index(name)
        if pos < 4: thisls=lslist[pos]        
        else: thisls=(45,(55,20))
            
        CumulativeReturns = ((1+retgroup[name]).cumprod()-1)*100.0
        CumulativeReturns.plot(label=ectranslate(name),ls=thisls)
    #plt.axhline(y=0,ls=":",c="red")
    plt.legend()
    plt.title(titletxt); plt.ylabel(ylabeltxt+'%'); plt.xlabel(xlabeltxt)
    plt.show()
    
    return

#==============================================================================
if __name__=='__main__':
    Market={'Market':('US','^GSPC')}
    Stocks={'BABA':.5,'JD':.5}
    portfolio=dict(Market,**Stocks)
    
    today='2021-1-28'
    pastyears=1

def portfolio_cumret(portfolio,today,pastyears=1):
    """
    功能：绘制投资组合的累计收益率趋势图，并与等权和市值加权组合比较
    """
    print("\nSearching for portfolio info, which may take time ...")
    # 解构投资组合
    _,_,tickerlist,sharelist=decompose_portfolio(portfolio)
    totalshares=round(sum(sharelist),4)
    if totalshares != 1.0:
        print("\n  #Error(portfolio_cumret): total weights is",totalshares,",it should be 1.0")
        return None       
    
    # 计算历史数据的开始日期
    start=get_start_date(today,pastyears)
    # 抓取股价
    prices=get_prices(tickerlist,start,today)
    if prices is None:
        print("\n#Error(portfolio_cumret): failed to get prices")
        print("  Possible reasons:")
        print("  1) incorrect stock code")
        print("  2) or unstable internet connection")
        return None
    if len(prices) == 0:
        print("\n#Error(portfolio_cumret): retrieved empty prices")
        print("  Possible reasons:")
        print("  1) stock code incorrect, delisted or suspended")
        print("  2) too slow and/or unstable internet connection")
        return None
    
    # 复权后收盘价
    aclose=prices['Close']    
    #print(aclose.head())
    # 计算每日收益率，并丢弃缺失值
    StockReturns = aclose.pct_change().dropna()
    if len(StockReturns) == 0:
        print("\n#Error(portfolio_cumret): retrieved empty returns")
        print("  Possible reasons:")
        print("  1) stock code incorrect, delisted or suspended")
        print("  2) too slow and/or unstable internet connection")
        return None
    #print(StockReturns.head())
    # 将收益率数据拷贝到新的变量 stock_return 中，为了后续调用的方便
    stock_return = StockReturns.copy()
    #..........................................................................
    
    # 设置组合权重，存储为numpy数组类型
    import numpy as np
    portfolio_weights = np.array(sharelist)
    # 计算portfolio的股票收益
    WeightedReturns = stock_return.mul(portfolio_weights, axis=1)
    # 计算投资组合的收益
    StockReturns['Portfolio'] = WeightedReturns.sum(axis=1)
    
    # 绘制portfolio随时间变化的图
    #仅用于绘图，以便使用收益率%来显示
    plotsr = StockReturns['Portfolio']*100.0
    #plotsr.plot(label='Portfolio')
    plotsr.plot(label='投资组合')
    plt.axhline(y=0,ls=":",c="red")
    plt.title("投资组合: 日收益率的变化趋势")
    plt.ylabel("日收益率%")
    plt.xlabel("数据来源: 雅虎财经, "+str(today))
    plt.legend()
    plt.show()
    #..........................................................................
    
    # 计算累积的组合收益，并绘图
    name_list=["Portfolio"]
    titletxt="投资组合: 累计收益率的变化趋势"
    ylabeltxt="累计收益率"
    xlabeltxt="数据来源: 雅虎财经, "+str(today)
    #由函数去处理百分号放大
    cumulative_returns_plot(StockReturns,name_list,titletxt,ylabeltxt,xlabeltxt)
    #..........................................................................
    
    # 设置投资组合中股票的数目
    numstocks = len(tickerlist)
    # 平均分配每一项的权重
    portfolio_weights_ew = np.repeat(1/numstocks, numstocks)
    # 计算等权重组合的收益
    StockReturns['Portfolio_EW']=stock_return.mul(portfolio_weights_ew,axis=1).sum(axis=1)
    #name_list=['Portfolio','Portfolio_EW']
    #cumulative_returns_plot(StockReturns,name_list,titletxt,ylabeltxt,xlabeltxt)
    #..........................................................................
    
    # 创建流通股市值Outstanding Market Capitalization
    MCap=prices['Close']*prices['Volume']
    #MCaplist=MCap.iloc[-1]
    MCaplist=MCap.mean(axis=0)    #求列的均值
    market_capitalizations = np.array(MCaplist)
    # 计算流通股市值权重
    mcap_weights = market_capitalizations / np.sum(market_capitalizations)
    # 计算市值加权的组合收益
    StockReturns['Portfolio_OMCap'] = stock_return.mul(mcap_weights, axis=1).sum(axis=1)

    #绘制累计收益率对比曲线
    name_list=['Portfolio', 'Portfolio_EW', 'Portfolio_OMCap']
    titletxt="投资组合: 累计收益率的比较"
    cumulative_returns_plot(StockReturns,name_list,titletxt,ylabeltxt,xlabeltxt)

    return [tickerlist,sharelist,StockReturns,stock_return,today, \
            [portfolio_weights,portfolio_weights_ew,mcap_weights]]

if __name__=='__main__':
    X=portfolio_cumret(portfolio,'2021-1-28')

#==============================================================================
def portfolio_corr(pf_info):
    """
    功能：绘制投资组合成分股之间相关关系的热力图
    """
    [_,_,StockReturns,stock_return,today,_]=pf_info
        
    sr=stock_return.copy()
    collist=list(sr)
    for col in collist:
        sr.rename(columns={col:codetranslate(col)},inplace=True)

    # 计算相关矩阵
    #correlation_matrix = stock_return.corr()
    correlation_matrix = sr.corr()
    
    # 导入seaborn
    import seaborn as sns
    # 创建热图
    sns.heatmap(correlation_matrix,annot=True,cmap="YlGnBu",linewidths=0.3,
            annot_kws={"size": 8})
    plt.title("投资组合: 成分股收益率之间的相关系数")
    plt.ylabel("成分股票")
    plt.xlabel("数据来源: 雅虎财经, "+str(today))
    plt.xticks(rotation=90); plt.yticks(rotation=0) 
    plt.show()

    return    

#==============================================================================
def portfolio_covar(pf_info):
    """
    功能：计算投资组合成分股之间的协方差
    """
    [_,_,StockReturns,stock_return,today,_]=pf_info

    # 计算协方差矩阵
    cov_mat = stock_return.cov()
    # 年化协方差矩阵，252个交易日
    cov_mat_annual = cov_mat * 252
    
    # 导入seaborn
    import seaborn as sns
    # 创建热图
    sns.heatmap(cov_mat_annual,annot=True,cmap="YlGnBu",linewidths=0.3,
            annot_kws={"size": 8})
    plt.title("投资组合: 成分股之间的协方差")
    plt.ylabel("成分股票")
    
    plt.xlabel("数据来源: 雅虎财经, "+str(today))
    plt.xticks(rotation=90)
    plt.yticks(rotation=0) 
    plt.show()

    return 

#==============================================================================
def portfolio_expectation(pf_info):
    """
    功能：计算投资组合的标准差
    """
    [tickerlist,sharelist,StockReturns,stock_return,today,[portfolio_weights,_,_]]=pf_info
    
    #取出观察期
    hstart0=StockReturns.index[0]
    hstart=str(hstart0.date())
    hend0=StockReturns.index[-1]
    hend=str(hend0.date())

    mean_return=stock_return.mul(portfolio_weights,axis=1).sum(axis=1).mean()
    annual_return = (1 + mean_return)**252 - 1
    
    # 计算协方差矩阵
    cov_mat = stock_return.cov()
    # 年化协方差矩阵
    cov_mat_annual = cov_mat * 252

    # 计算投资组合的标准差
    import numpy as np
    portfolio_volatility = np.sqrt(np.dot(portfolio_weights.T, 
                                      np.dot(cov_mat_annual, portfolio_weights)))
    print("\n===== 投资组合的预期收益与预期风险 =====")
    print("投资组合:")
    print("成分股：",tickerlist)
    print("权重:",sharelist)
    print("观察期:",hstart+'至'+hend)
    print("预期收益:",round(annual_return,4))
    print("预期风险:",round(portfolio_volatility,4))
    
    #import datetime as dt; today=dt.date.today()    
    print("*数据来源：雅虎财经，"+str(today))

    return 

#==============================================================================


def portfolio_es(pf_info,simulation=1000):
    """
    功能：计算投资组合的标准差，绘制投资组合的有效集
    """
    [tickerlist,_,StockReturns,stock_return,today,_]=pf_info
    #通过列名列表获得成分股个数
    numstocks=len(tickerlist)

    # 计算协方差矩阵
    cov_mat = stock_return.cov()
    # 年化协方差矩阵
    cov_mat_annual = cov_mat * 252

    # 设置模拟的次数
    number = simulation
    # 设置空的numpy数组，用于存储每次模拟得到的成分股权重、组合的收益率和标准差
    import numpy as np
    random_p = np.empty((number,numstocks+2))
    # 设置随机数种子，这里是为了结果可重复
    np.random.seed(123)

    # 循环模拟n次随机的投资组合
    print("\nCalculating portfolio efficient sets, please wait ...")    
    for i in range(number):
        # 生成numstocks个随机数，并归一化，得到一组随机的权重数据
        random9 = np.random.random(numstocks)
        random_weight = random9 / np.sum(random9)
    
        # 计算年化平均收益率
        mean_return=stock_return.mul(random_weight,axis=1).sum(axis=1).mean()
        annual_return = (1 + mean_return)**252 - 1

        # 计算年化的标准差，也称为波动率
        random_volatility=np.sqrt(np.dot(random_weight.T, 
                                np.dot(cov_mat_annual,random_weight)))

        # 将上面生成的权重，和计算得到的收益率、标准差存入数组random_p中
        random_p[i][:numstocks] = random_weight
        random_p[i][numstocks] = annual_return
        random_p[i][numstocks+1] = random_volatility
    
    # 将numpy数组转化成DataFrame数据框
    import pandas as pd
    RandomPortfolios = pd.DataFrame(random_p)
    # 设置数据框RandomPortfolios每一列的名称
    RandomPortfolios.columns = [ticker + "_weight" for ticker in tickerlist]  \
                         + ['Returns', 'Volatility']

    # 绘制散点图
    RandomPortfolios.plot('Volatility','Returns',kind='scatter',color='y',edgecolors='k')
    """
    plt.style.use('seaborn-dark')
    RandomPortfolios.plot.scatter(x='Volatility', y='Returns', c='Returns',
                cmap='RdYlGn', edgecolors='black')
    """
    plt.title("投资组合: 马科维茨可行集")
    plt.ylabel("预期收益")
    plt.xlabel("预期风险-->\n数据来源: 雅虎财经, "+str(today))
    plt.show()

    return [pf_info,RandomPortfolios]

#==============================================================================
def portfolio_GMV(es_info):
    """
    功能：计算投资组合的最小风险组合
    GMV=Global Minimium Variance
    """
    [[tickerlist,sharelist,StockReturns,stock_return,today, \
          [portfolio_weights,portfolio_weights_ew,mcap_weights]], \
          RandomPortfolios]=es_info
    numstocks=len(tickerlist)
    
    #取出观察期
    hstart0=StockReturns.index[0]
    hstart=str(hstart0.date())
    hend0=StockReturns.index[-1]
    hend=str(hend0.date())
        
    # 绘制散点图
    RandomPortfolios.plot('Volatility','Returns',kind='scatter',color='y',edgecolors='k')

    # 找到标准差最小数据的索引值
    min_index = RandomPortfolios.Volatility.idxmin()
    # 在收益-风险散点图中突出风险最小的点
    x = RandomPortfolios.loc[min_index,'Volatility']
    y = RandomPortfolios.loc[min_index,'Returns']
    plt.scatter(x, y, color='m',marker='8',s=100,label="GMV Point",edgecolors='r')   

    plt.title("投资组合: GMV点的位置")
    plt.ylabel("预期收益")
    plt.xlabel("预期风险-->\nGMV表示风险最低的组合方式(Global Minimium Variance). \
               \n数据来源: 雅虎财经, "+str(today))
    plt.legend()
    plt.show()

    # 提取最小波动组合对应的权重, 并转换成Numpy数组
    import numpy as np
    GMV_weights = np.array(RandomPortfolios.iloc[min_index, 0:numstocks])
    # 计算GMV投资组合收益
    StockReturns['Portfolio_GMV'] = stock_return.mul(GMV_weights, axis=1).sum(axis=1)
    # 绘制累积收益曲线
    namelist=['Portfolio_GMV', 'Portfolio']
    titletxt="投资组合: 累计收益率的比较"
    ylabeltxt="累计收益率"
    xlabeltxt="数据来源: 雅虎财经, "+str(today)    
    cumulative_returns_plot(StockReturns,namelist,titletxt,ylabeltxt,xlabeltxt)

    #输出GMV投资组合构成比例
    print("\n===== GMV组合方式: 构造风险最低的投资组合 =====")
    print("成分股：",tickerlist)
    GMV_weights_new=ratiolist_round(GMV_weights,num=3)
    print("权重：",GMV_weights_new)
    print("观察期:",hstart+'至'+hend)
    print("预期收益:",round(y,4))
    print("预期风险:",round(x,4))

    import datetime as dt; today=dt.date.today()    
    print("*数据来源：雅虎财经，"+str(today))

    return

#==============================================================================
def portfolio_MSR_GMV(es_info,RF=0):
    """
    功能：计算投资组合的最高夏普比率组合
    MSR=Maximium Sharpe Rate
    """
    [[tickerlist,sharelist,StockReturns,stock_return,today, \
          [portfolio_weights,portfolio_weights_ew,mcap_weights]], \
          RandomPortfolios]=es_info
    numstocks=len(tickerlist)    
    
    #取出观察期
    hstart0=StockReturns.index[0]
    hstart=str(hstart0.date())
    hend0=StockReturns.index[-1]
    hend=str(hend0.date())
    
    # 设置无风险回报率
    risk_free = RF
    # 计算每项资产的夏普比率
    RandomPortfolios['Sharpe'] = (RandomPortfolios.Returns - risk_free)   \
                            / RandomPortfolios.Volatility

    # 绘制收益-标准差的散点图，并用颜色描绘夏普比率
    # 绘制散点图
    RandomPortfolios.plot('Volatility','Returns',kind='scatter',alpha=0.3)
    plt.scatter(RandomPortfolios.Volatility, RandomPortfolios.Returns, 
            c=RandomPortfolios.Sharpe)
    plt.colorbar(label='Colored in Sharpe Ratio')
    plt.title("投资组合: 马科维茨可行集的夏普比率分布")
    plt.ylabel("预期收益")
    plt.xlabel("预期风险-->"+ \
               "\n观察期："+hstart+"至"+hend+ \
               "\n数据来源: 雅虎财经, "+str(today))     
    plt.show()

    #绘制有效集
    RandomPortfolios.plot('Volatility','Returns',kind='scatter',color='y',edgecolors='k')

    # 找到夏普比率最大数据对应的索引值
    max_index = RandomPortfolios.Sharpe.idxmax()
    # 在收益-风险散点图中突出夏普比率最大的点
    MSR_x = RandomPortfolios.loc[max_index,'Volatility']
    MSR_y = RandomPortfolios.loc[max_index,'Returns']
    plt.scatter(MSR_x, MSR_y, color='red',marker='*',s=150,label="MSR Point")  
    # 提取最大夏普比率组合对应的权重，并转化为numpy数组
    import numpy as np    
    MSR_weights = np.array(RandomPortfolios.iloc[max_index, 0:numstocks])
    # 计算MSR组合的收益
    StockReturns['Portfolio_MSR'] = stock_return.mul(MSR_weights, axis=1).sum(axis=1)
    
    # 找到标准差最小数据的索引值
    min_index = RandomPortfolios.Volatility.idxmin()
    # 提取最小波动组合对应的权重, 并转换成Numpy数组
    # 在收益-风险散点图中突出风险最小的点
    GMV_x = RandomPortfolios.loc[min_index,'Volatility']
    GMV_y = RandomPortfolios.loc[min_index,'Returns']
    plt.scatter(GMV_x, GMV_y, color='m',marker='8',s=100,label="GMV Point") 
    # 提取最小风险组合对应的权重，并转化为numpy数组
    GMV_weights = np.array(RandomPortfolios.iloc[min_index, 0:numstocks])
    # 计算GMV投资组合收益
    StockReturns['Portfolio_GMV'] = stock_return.mul(GMV_weights, axis=1).sum(axis=1)

    plt.title("投资组合: MSR点和GMV点的位置")
    plt.ylabel("预期收益")
    plt.xlabel("预期风险-->"+ \
               "\n观察期："+hstart+"至"+hend+ \
               "\n数据来源: 雅虎财经, "+str(today))    
    plt.legend()
    plt.show()

    # 绘制累积收益曲线
    namelist=['Portfolio_MSR','Portfolio_GMV','Portfolio']
    titletxt="投资组合: 累计收益率的比较"
    ylabeltxt="累计收益率"
    xlabeltxt="观察期："+hstart+"至"+hend+ \
              "\n数据来源: 雅虎财经, "+str(today)      
    cumulative_returns_plot(StockReturns,namelist,titletxt,ylabeltxt,xlabeltxt)    

    #输出MSR投资组合构成比例
    print("\n===== 投资组合的构造方式: 最高夏普比率(MSR) =====")
    print("成分股：",translate_tickerlist(tickerlist))
    MSR_weights_new=ratiolist_round(MSR_weights,3)
    print("权重：",MSR_weights_new)
    print("观察期:",hstart+"至"+hend)
    print("预期收益:",round(MSR_y*100.0,2),'\b%')
    print("预期风险:",round(MSR_x*100.0,2),'\b%')

    #import datetime as dt; today=dt.date.today()    
    print("*数据来源：雅虎财经，"+str(today))

    #输出GMV投资组合构成比例
    print("\n===== 投资组合的构造方式: 风险最小化(GMV) =====")
    print("成分股：",translate_tickerlist(tickerlist))
    GMV_weights_new=ratiolist_round(GMV_weights,3)
    print("权重：",GMV_weights_new)
    print("观察期:",hstart+"至"+hend)
    print("预期收益:",round(GMV_y*100.0,2),'\b%')
    print("预期风险:",round(GMV_x*100.0,2),'\b%')    
    print("*数据来源：雅虎财经，"+str(today))
    
    return StockReturns

#==============================================================================
def translate_tickerlist(tickerlist):
    newlist=[]
    for t in tickerlist:
        name=codetranslate(t)
        newlist=newlist+[name]
        
    return newlist
#==============================================================================
# 绘制马科维茨有效边界
#==============================================================================
def ret_monthly(ticker,prices): 
    """
    功能：
    """
    price=prices['Adj Close'][ticker]
    
    import numpy as np
    div=price.pct_change()+1
    logret=np.log(div)
    import pandas as pd
    lrdf=pd.DataFrame(logret)
    lrdf['ymd']=lrdf.index.astype("str")
    lrdf['ym']=lrdf['ymd'].apply(lambda x:x[0:7])
    lrdf.dropna(inplace=True)
    
    mret=lrdf.groupby(by=['ym'])[ticker].sum()
    
    return mret

if __name__=='__main__':
    ticker='MSFT'
    fromdate,todate='2019-1-1','2020-8-1'

#==============================================================================
def objFunction(W,R,target_ret):
    
    import numpy as np
    stock_mean=np.mean(R,axis=0)
    port_mean=np.dot(W,stock_mean) # portfolio mean
    
    cov=np.cov(R.T) # var-cov matrix
    port_var=np.dot(np.dot(W,cov),W.T) # portfolio variance
    penalty = 2000*abs(port_mean-target_ret)# penalty 4 deviation
    
    objfunc=np.sqrt(port_var) + penalty # objective function 
    
    return objfunc   

#==============================================================================
def portfolio_ef_0(stocks,fromdate,todate):
    """
    功能：绘制马科维茨有效前沿，不区分上半沿和下半沿
    """
    #Code for getting stock prices
    prices=get_prices(stocks,fromdate,todate)
    
    #Code for generating a return matrix R
    R0=ret_monthly(stocks[0],prices) # starting from 1st stock
    n_stock=len(stocks) # number of stocks
    import pandas as pd
    import numpy as np
    for i in range(1,n_stock): # merge with other stocks
        x=ret_monthly(stocks[i],prices)
        R0=pd.merge(R0,x,left_index=True,right_index=True)
        R=np.array(R0)    

    #Code for estimating optimal portfolios for a given return
    out_mean,out_std,out_weight=[],[],[]
    import numpy as np
    stockMean=np.mean(R,axis=0)
    
    from scipy.optimize import minimize
    for r in np.linspace(np.min(stockMean),np.max(stockMean),num=100):
        W = np.ones([n_stock])/n_stock # starting from equal weights
        b_ = [(0,1) for i in range(n_stock)] # bounds, here no short
        c_ = ({'type':'eq', 'fun': lambda W: sum(W)-1. }) #constraint
        result=minimize(objFunction,W,(R,r),method='SLSQP'
                                    ,constraints=c_, bounds=b_)
        if not result.success: # handle error raise    
            BaseException(result.message)
        
        out_mean.append(round(r,4)) # 4 decimal places
        std_=round(np.std(np.sum(R*result.x,axis=1)),6)
        out_std.append(std_)
        out_weight.append(result.x)

    #Code for plotting the efficient frontier
    
    plt.title('Efficient Frontier of Portfolio')
    plt.xlabel('Standard Deviation of portfolio (Risk))')
    plt.ylabel('Return of portfolio')
    
    out_std_min=min(out_std)
    pos=out_std.index(out_std_min)
    out_mean_min=out_mean[pos]
    x_left=out_std_min+0.25
    y_left=out_mean_min+0.5
    
    #plt.figtext(x_left,y_left,str(n_stock)+' stock are used: ')
    plt.figtext(x_left,y_left,"投资组合由"+str(n_stock)+'种证券构成: ')
    plt.figtext(x_left,y_left-0.05,' '+str(stocks))
    plt.figtext(x_left,y_left-0.1,'观察期间：'+str(fromdate)+'至'+str(todate))
    plt.plot(out_std,out_mean,color='r',ls=':',lw=4)
    plt.show()    
    
    return

if __name__=='__main__':
    stocks=['IBM','WMT','AAPL','C','MSFT']
    fromdate,todate='2019-1-1','2020-8-1'    

#==============================================================================
def portfolio_ef(stocks,fromdate,todate):
    """
    功能：多只股票的马科维茨有效边界，区分上半沿和下半沿，标记风险极小点
    """
    print("\n...Searching for portfolio information, please wait...")
    #Code for getting stock prices
    prices=get_prices(stocks,fromdate,todate)
    
    #Code for generating a return matrix R
    R0=ret_monthly(stocks[0],prices) # starting from 1st stock
    n_stock=len(stocks) # number of stocks
    
    import pandas as pd
    import numpy as np
    for i in range(1,n_stock): # merge with other stocks
        x=ret_monthly(stocks[i],prices)
        R0=pd.merge(R0,x,left_index=True,right_index=True)
        R=np.array(R0)    

    #Code for estimating optimal portfolios for a given return
    out_mean,out_std,out_weight=[],[],[]
    stockMean=np.mean(R,axis=0)
    
    from scipy.optimize import minimize
    for r in np.linspace(np.min(stockMean),np.max(stockMean),num=100):
        W = np.ones([n_stock])/n_stock # starting from equal weights
        b_ = [(0,1) for i in range(n_stock)] # bounds, here no short
        c_ = ({'type':'eq', 'fun': lambda W: sum(W)-1. }) #constraint
        result=minimize(objFunction,W,(R,r),method='SLSQP'
                                    ,constraints=c_, bounds=b_)
        if not result.success: # handle error raise    
            BaseException(result.message)
        
        out_mean.append(round(r,4)) # 4 decimal places
        std_=round(np.std(np.sum(R*result.x,axis=1)),6)
        out_std.append(std_)
        out_weight.append(result.x)

    #Code for positioning
    out_std_min=min(out_std)
    pos=out_std.index(out_std_min)
    out_mean_min=out_mean[pos]
    x_left=out_std_min+0.25
    y_left=out_mean_min+0.5
    
    import pandas as pd
    out_df=pd.DataFrame(out_mean,out_std,columns=['mean'])
    out_df_ef=out_df[out_df['mean']>=out_mean_min]
    out_df_ief=out_df[out_df['mean']<out_mean_min]

    #Code for plotting the efficient frontier
    
    plt.title('投资组合：马科维茨有效边界')
    
    import datetime as dt; today=dt.date.today()    
    plt.xlabel('预期风险-->'+"\n数据来源：雅虎财经, "+str(today))
    plt.ylabel('预期收益')
    
    plt.figtext(x_left,y_left,"投资组合由"+str(n_stock)+'种证券构成: ')
    plt.figtext(x_left,y_left-0.05,' '+str(stocks))
    plt.figtext(x_left,y_left-0.1,'观察期间：'+str(fromdate)+'至'+str(todate))
    plt.plot(out_df_ef.index,out_df_ef['mean'],color='r',ls='--',lw=2,label='有效边界')
    plt.plot(out_df_ief.index,out_df_ief['mean'],color='k',ls=':',lw=2,label='无效边界')
    plt.plot(out_std_min,out_mean_min,'g*-',markersize=16,label='风险最低点')
    
    plt.legend(loc='best')
    plt.show()    
    
    return out_df

if __name__=='__main__':
    stocks=['IBM','WMT','AAPL','C','MSFT']
    fromdate,todate='2019-1-1','2020-8-1'    





































# -*- coding: utf-8 -*-
"""
本模块功能：宏观经济分析
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2020年8月31日
最新修订日期：2020年8月31日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""
#==============================================================================
#关闭所有警告
import warnings; warnings.filterwarnings('ignore')
from siat.grafix import *
from siat.common import *
from siat.translate import *
#==============================================================================
if __name__=='__main__':
    start='2010-1-1'; end='2020-12-31'
    scope='China'; factor='Constant GDP'


def get_econ_factors(start,end,scope='China',factor='GDP'):
   """
   与函数get_econ_factors0的区别：
   1、将GNP替换为GNI
   2、将GNP Ratio替换为GNI Ration
   3、新增Currency Value，货币价值，衡量货币贬值速度
   """
   
   #替换GNP为GNI
   if factor == 'GNP':
       factor='GNI'
       ds=get_econ_factors0(start,end,scope,factor)
       return ds

   #替换GNP Ratio为GNI Ratio
   if factor in ['GNP Ratio','GNI Ratio']:
       ds_gni=get_econ_factors0(start,end,scope,'GNI')  
       if ds_gni is None: 
           print("  #Error(get_econ_factors): info not found for",scope,'GNI')
           return None
       
       import pandas as pd
       ds_gni1=pd.DataFrame(ds_gni['VALUE'])
       
       ds_gdp=get_econ_factors0(start,end,scope,'GDP') 
       if ds_gdp is None: return None
       
       ds_gdp1=pd.DataFrame(ds_gdp['VALUE'])
       
       ds1=pd.merge(ds_gni1,ds_gdp1,how='inner',left_index=True,right_index=True)
       ds1.dropna(inplace=True)
       ds1['VALUE']=ds1['VALUE_x']/ds1['VALUE_y']
       ds2=ds1.drop(['VALUE_x','VALUE_y'],axis=1)
       
       ds_gdp2=ds_gdp.drop('VALUE',axis=1)
       ds_gdp2['factor']='GNI Ratio'
       ds_gdp2['name']='GNI/GDP Ratio'
       ds_gdp2['symbol']='CUSTOMIZED'
       
       ds3=pd.merge(ds2,ds_gdp2,how='inner',left_index=True,right_index=True)
       
       return ds3

   #新增Currency Value
   if factor in ['Currency Value']:
       cv=get_econ_factors0(start,end,scope,'Constant CPI') 
       if cv is None: 
           print("  #Error(get_econ_factors): info not found for",scope,'Constant CPI')
           return None
       
       cv['CV']=(100.0/cv['VALUE'])*100.0
       cv1=cv.rename(columns={'VALUE':'Constant CPI'})
       cv2=cv1.rename(columns={'CV':'VALUE'})
       
       cv2['factor']='Currency Value'
       cv2['name']='Currency Purchasing Power Based on CPI'
       
       return cv2

   #其他正常情况
   df=get_econ_factors0(start,end,scope=scope,factor=factor)
    
   return df

#==============================================================================
def get_econ_factors0(start,end,scope='China',factor='GDP'):
   """
   【支持的因子种类(factor)】
   GDP, CPI, PPI,...

   【支持的国家/地区(scope)】
   US: 美国
   China: 中国
   Korea: 韩国
   Japan: 日本
   India: 印度

   【支持的取样频率(freq)】
   Annual: 年
   Quarterly: 季度
   Monthly: 月
   """
   s=fred_factor_codes() 

   #帮助1：国家列表
   if scope=='?':
       scopelist=list(set(list(s[s['factor']==factor]['scope'])))
       if len(scopelist)==0:
           print("  Error(get_econ_factors): no such economy factor,",factor)
       else:
           title="\n*** Supported scopes for factor, "+factor+':'
           print(title)
           linelen=0
           for i in range(len(scopelist)):
               print(scopelist[i],end='')
               linelen=linelen+len(scopelist[i])+2
               if (linelen >= len(title)-2): 
                   print(''); linelen=0
               else:
                   if (i+1) < len(scopelist): 
                       print(', ',end='')
                   else:
                       print('')
       return None
   
   #帮助2：经济指标
   if (scope != '?') and (factor=='?'):
       factorlist=list(set(list(s[s['scope']==scope]['factor'])))
       if len(factorlist)==0:
           print("  Error(get_econ_factors): no or unavailable country/region,",scope)
       else:
           title="\n*** Supported factors for scope, "+scope+':'
           print(title)
           linelen=0
           for i in range(len(factorlist)):
               print(factorlist[i],end='')
               linelen=linelen+len(factorlist[i])+2
               if (linelen >= len(title)-2): 
                   print(''); linelen=0
               else:
                   if (i+1) < len(factorlist): 
                       print(', ',end='')
                   else:
                       print('')
       return None

   #匹配：scope+factor+freq
   ss=s[s['scope'].isin([scope]) & s['factor'].isin([factor])]  
   #如果未找到匹配的模式，显示信息后返回
   if len(ss)==0:
        print("  Error(get_econ_factors): info are not available for",scope,factor)
        return None

   #取出对应的关键字symbol
   symbol=list(ss['symbol'])[0]

   #按照关键字抓取数据
   import pandas_datareader as web
   try:
        ds=web.get_data_fred(symbol,start,end)
   except:
        print("  #Error(get_econ_factors): connection to data source failed!")        
        return None
   if len(ds)==0:
        print("  #Error(get_econ_factors): server returned empty data!")        
        return None
   # 结果字段统一改名
   ds.columns=['VALUE'] 
   ds['scope']=list(ss['scope'])[0]
   ds['factor']=list(ss['factor'])[0]
   ds['freq']=list(ss['freq'])[0]
   ds['name']=list(ss['name'])[0]
   ds['symbol']=list(ss['symbol'])[0]
   ds['units']=list(ss['units'])[0]

   return ds

if __name__=='__main__':
    start='2010-1-1'; end='2020-8-31'
    scope='China'; factor='Real GDP Per Capita'
    
    ds1=get_econ_factors('2010-1-1','2020-8-31','China','GDP')
    ds2=get_econ_factors('2010-1-1','2020-8-31','China','Real GDP')
    ds3=get_econ_factors('2010-1-1','2020-8-31','India','Constant GDP')
    get_econ_factors('2010-1-1','2020-8-31','?','Constant GDP')
    get_econ_factors('2010-1-1','2020-8-31','China','?')
#==============================================================================
def economy_trend(start,end,scope='China',factor='GDP',datatag=False,power=3,zeroline=False):
    """
    功能：绘制宏观经济指标，单线，有趋势线    
    """
    #检查日期期间的合理性
    valid,_,_=check_period(start,end)
    if not valid:
        print('  Error(trend_economy): period not valid:',start,end)
        return
    
    #获取指标
    ds=get_econ_factors(start,end,scope,factor)
    if (ds is None):
        print('  #Error(economy_trend): scope/economic factor not available:',scope,'/',factor)
        return

    #绘图
    ylabeltxt=ectranslate(factor)
    titletxt=ectranslate(list(ds['name'])[0])+'的走势：'+ectranslate(scope)
    
    import datetime
    today=datetime.date.today()
    footnote='单位: '+list(ds['units'])[0]+', '+list(ds['freq'])[0]+ \
        '\n数据来源: OECD|IMF|WB|FRED, '+str(today)
    ds.dropna(inplace=True)
    plot_line(ds,'VALUE',ectranslate(factor),ylabeltxt,titletxt,footnote,datatag,power=power,zeroline=zeroline)
    
    return ds


if __name__=='__main__':
    start='2010-1-1'; end='2020-8-31'
    scope='USA'; factor='Real GDP Per Capita'
    
    ds=economy_trend(start,end,scope='China',factor='GDP')
    ds=economy_trend(start,end,scope='Japan',factor='Real GDP')

#==============================================================================
def compare_economy(tickers,measures,fromdate,todate,power=0,twinx=False):
    """
    功能：对比绘制折线图：一个国家的两种测度，或两个国家的同一个测度。
    输入：
    国家代码tickers，如果是一个列表且内含两个代码，则认为希望比较两个国家的
    同一个测度指标。如果是一个列表但只内含一个国家代码或只是一个国家代码的字符串，
    则认为希望比较一个国家的两个测度指标。
    测度指标measures：如果是一个列表且内含两个测度指标，则认为希望比较一个国家的
    两个测度指标。如果是一个列表但只内含一个测度指标或只是一个测度指标的字符串，
    则认为希望比较两个国家的同一个测度指标。
    如果两个判断互相矛盾，以第一个为准。
    开始日期fromdate，结束日期todate。
    输出：绘制经济指标折线图，手动指定是否使用单轴或双轴坐标。
    返回：无
    """
    #检查日期期间的合理性
    valid,_,_=check_period(fromdate,todate)
    if not valid:
        print('  Error(compare_economy): period not valid:',fromdate,todate)
        return
    
    #判断国家代码个数
    #如果tickers只是一个字符串
    security_num = 0
    if isinstance(tickers,str): 
        security_num = 1
        ticker1 = tickers
    #如果tickers是一个列表
    if isinstance(tickers,list): 
        security_num = len(tickers)
        if security_num == 0:
            print("  Error(compare_economy): 未提供国家代码！")
            return
        if security_num >= 1: ticker1 = tickers[0]
        if security_num >= 2: ticker2 = tickers[1]
            
    #判断测度个数
    #如果measures只是一个字符串
    measure_num = 0
    if isinstance(measures,str): 
        measure_num = 1
        measure1 = measures
    #如果measures是一个列表
    if isinstance(measures,list): 
        measure_num = len(measures)
        if measure_num == 0:
            print("  Error(compare_economy): 未提供测度指标！")
            return
        if measure_num >= 1: measure1 = measures[0]
        if measure_num >= 2: measure2 = measures[1]

    import datetime; today=datetime.date.today()
    #是否单一国家代码+两个测度指标
    if (security_num == 1) and (measure_num >= 2):
        #国家ticker1：抓取经济指标measure1
        df1=get_econ_factors(fromdate,todate,ticker1,measure1)
        if df1 is None: return None, None

        #国家ticker1：抓取经济指标measure2
        df2=get_econ_factors(fromdate,todate,ticker1,measure2)
        if df2 is None: return None, None
        
        #绘制单个国家的双指标对比图
        ylabeltxt=''
        titletxt=ectranslate(measure1)+' vs '+ectranslate(measure2)+ \
            '\n（'+ectranslate(ticker1)+'）'
            
        footnote=ectranslate(measure1)+': '+list(df1['units'])[0]+', '+list(df1['freq'])[0]+ \
            '\n'+ectranslate(measure2)+': '+list(df2['units'])[0]+', '+list(df2['freq'])[0]+ \
            '\n数据来源: OECD|IMF|WB|FRED, '+str(today)
            
        plot_line2(df1,ectranslate(ticker1),'VALUE',ectranslate(measure1), \
               df2,ectranslate(ticker1),'VALUE',ectranslate(measure2), \
               ylabeltxt,titletxt,footnote, \
               power=power,twinx=twinx)        
        
    elif (security_num >= 2) and (measure_num >= 1):
        #双国家+单个测度指标        
        #国家ticker1：抓取经济指标measure1
        df1=get_econ_factors(fromdate,todate,ticker1,measure1)
        if df1 is None: return None, None

        #国家ticker2：抓取经济指标measure1
        df2=get_econ_factors(fromdate,todate,ticker2,measure1)
        if df2 is None: return None, None

        #绘制双国家单指标对比图
        ylabeltxt=ectranslate(measure1)
        #这里的GNP指标实际上是GNP vs GDP的百分比
        yline=999   #默认不绘制水平线
        if measure1 in ['GNP Ratio','GNI Ratio']:
            ylabeltxt='GNP(GNI)/GDP'
            yline=1 #绘制y=1的水平线
        if measure1 in ['Currency Value','Constant CPI']:
            ylabeltxt=ectranslate(measure1)
            yline=100 #绘制y=100的水平线

        titletxt=ectranslate(list(df1['name'])[0])+'走势对比：'+ \
            '\n'+ectranslate(ticker1)+' vs '+ectranslate(ticker2)
        
        footnote='单位: '+list(df1['units'])[0]+', '+list(df1['freq'])[0]+ \
            '\n数据来源: OECD|IMF|WB|FRED, '+str(today)
            
        plot_line2(df1,ectranslate(ticker1),'VALUE',ectranslate(measure1), \
               df2,ectranslate(ticker2),'VALUE',ectranslate(measure1), \
               ylabeltxt,titletxt,footnote, \
               power=power,twinx=twinx,yline=yline)         
    else:
        print("  Error(compare_economy): 未能理解需要比较的内容！")
        return None,None

    return df1,df2

if __name__ =="__main__":
    tickers=['China','India']
    measures='GDP'
    fromdate='2010-1-1'
    todate='2020-8-31'
    
    df=compare_economy(tickers,measures,fromdate,todate)
    df=compare_economy(tickers,measures,fromdate,todate,twinx=True)

    df=compare_economy(tickers,'GDP Per Capita',fromdate,todate)
    df=compare_economy(tickers,'GDP Per Capita',fromdate,todate,twinx=True)

    df=compare_economy('India',['Real GDP','Real GDP Per Capita'],fromdate,todate,twinx=True)
    df=compare_economy('Japan',['Real GDP','Real GDP Per Capita'],fromdate,todate,twinx=True)
    df=compare_economy('Israel',['Real GDP','Real GDP Per Capita'],fromdate,todate,twinx=True)

#==============================================================================
def econ_fin_depth(fromdate,todate,scope,power=0,graph=True):
    """
    功能：经济的金融化深度，一个国家
    """
    #检查日期期间的合理性
    valid,_,_=check_period(fromdate,todate)
    if not valid:
        print('  Error(econ_fin_depth): period not valid:',fromdate,todate)
        return None   
    
    #获取GDP，按季度，本币
    gdp_qtr=get_econ_factors(fromdate,todate,scope,'Constant GDP')
    gdp_qtr['date']=gdp_qtr.index.date
    datecvt=lambda x: str(x)[0:4]
    gdp_qtr['date_str']=gdp_qtr['date'].apply(datecvt)

    import numpy as np
    gdp_annual=gdp_qtr.groupby(['date_str'])['VALUE'].agg(['count',np.sum])
    gdp_annual2=gdp_annual[gdp_annual['count']==4]

    #获取M2，按月，本币
    m2_mth=get_econ_factors(fromdate,todate,scope,'M2')
    m2_mth['date']=m2_mth.index.date
    datecvt=lambda x: str(x)[0:4]
    m2_mth['date_str']=m2_mth['date'].apply(datecvt)

    m2_annual=m2_mth.groupby(['date_str'])['VALUE'].agg(['count',np.sum])
    m2_annual2=m2_annual[m2_annual['count']==12]

    #合并
    import pandas as pd
    m2_gdp=pd.merge(m2_annual2,gdp_annual2,on='date_str')
    m2_gdp.dropna(inplace=True)
    m2_gdp['m2/gdp']=m2_gdp['sum_x']/m2_gdp['sum_y']
    
    df=m2_gdp[['m2/gdp']]

    #绘图
    if not graph: return df
    colname='m2/gdp'
    collabel="经济的金融深度"
    ylabeltxt="M2/GDP比例"
    titletxt=ectranslate(scope)+': 经济的金融深度'
    footnote="数据来源: OECD|IMF|WB|FRED"
    plot_line(df,colname,collabel,ylabeltxt,titletxt,footnote,power=power)

    return df

if __name__=='__main__':
    fromdate='2000-1-1'; todate='2020-8-31'
    scope="China"
    cn=econ_fin_depth(fromdate,todate,scope,power=4)
    print(min(cn['m2/gdp']),max(cn['m2/gdp']))
    print(cn)
    jp=econ_fin_depth(fromdate,todate,'Japan',power=0)    
    us=econ_fin_depth(fromdate,todate,'USA',power=0)
    kr=econ_fin_depth(fromdate,todate,'Korea',power=0)

#==============================================================================
def compare_efd(fromdate,todate,scopelist,power=0):
    """
    功能：比较经济的金融化深度，两个国家
    """
    #检查日期期间的合理性
    valid,_,_=check_period(fromdate,todate)
    if not valid:
        print('  Error(econ_fin_depth): period not valid:',fromdate,todate)
        return None,None   

    #检查国家列表
    if isinstance(scopelist,list): 
        if len(scopelist) < 2:
            print("  Error(compare_efd): need a list with 2 countries",scopelist)
            return None,None
        scope1 = scopelist[0]; scope2 = scopelist[1]
    else:
        print("  Error(compare_efd): need a list with 2 countries",scopelist)
        return None,None
 
    #计算scope1/2的efd。美国的M2指标单位是Billions Dollars
    df1=econ_fin_depth(fromdate,todate,scope1,graph=False)
    if scope1=='USA':
        df1['m2/gdp']=df1['m2/gdp']*1000000000.0
    df2=econ_fin_depth(fromdate,todate,scope2,graph=False)
    if scope2=='USA':
        df2['m2/gdp']=df2['m2/gdp']*1000000000.0
    
    #绘图
    ticker1=scope1; ticker2=scope2
    colname1='m2/gdp'; colname2='m2/gdp'
    label1="M2/GDP"; label2="M2/GDP"
    ylabeltxt="M2/GDP比例"
    titletxt='经济金融深度走势对比'+'\n'+ectranslate(scope1)+' vs '+ectranslate(scope2)
    
    import datetime
    today=datetime.date.today()
    footnote="数据来源：IMF/FRED，"+str(today)    
    plot_line2(df1,ectranslate(ticker1),colname1,label1, \
               df2,ectranslate(ticker2),colname2,label2, \
               ylabeltxt,titletxt,footnote, \
               power=power)
    return df1,df2

if __name__=='__main__':
    fromdate='2000-1-1'; todate='2020-8-31'
    scopelist=["China","Japan"]
    power=4
    cn,us=compare_efd(fromdate,todate,['China','USA'])
    cn,jp=compare_efd(fromdate,todate,['China','Japan'])
    cn,kr=compare_efd(fromdate,todate,['China','Korea'])

#==============================================================================
def economy_security(scope,fromdate,todate,econ_factor,sec_ticker):
    """
    功能：比较宏观经济与证券市场之间的关联关系
    scope: 国家/地区
    econ_factor: 例如GDP
    sec_ticker: 例如标普500指数、道琼斯指数、上证综合指数
    输出： df
    """
    #检查日期期间的合理性
    valid,_,_=check_period(fromdate,todate)
    if not valid:
        print('  Error(economy_security): period not valid:',fromdate,todate)
        return None,None

    #获得econ_factor
    econ=get_econ_factors(fromdate,todate,scope,econ_factor)
    if econ is None:
        print('  Error(economy_security): scope/economic factor not available:',scope,econ_factor)
        return None,None
    econ_growth='Growth %'
    econ[econ_growth]=econ['VALUE'].pct_change()*100.0
    
    #获得sec_ticker行情
    import siat.security_prices as ssp
    sec=ssp.get_price(sec_ticker,fromdate,todate)
    if sec is None:
        print('  Error(economy_security): ticker info not available:',sec_ticker)
        return None,None    

    #绘图1： 直接指标，双轴
    econ_min1,econ_max1=get_df_period(econ)    
    sec_min1,sec_max1=get_df_period(sec)
    min1=max(econ_min1,sec_min1)
    max1=min(econ_max1,sec_max1)
    econ1=set_df_period(econ,min1,max1)
    sec1=set_df_period(sec,min1,max1)    
    
    df1=econ1; ticker1=econ_factor; colname1='VALUE'; label1=econ_factor
    df2=sec1; ticker2=sec_ticker; colname2='Close'; label2=sec_ticker
    ylabeltxt=''
    titletxt=ectranslate(scope)+": 宏观经济与证券市场的关系"
    footnote="数据来源: FRED|Yahoo Finance"
    plot_line2(df1,ectranslate(ticker1),colname1,label1, \
               df2,ectranslate(ticker2),colname2,label2, \
               ylabeltxt,titletxt,footnote, \
               power=0,twinx=True)

    #绘图2： 增长指标，双轴
    econ2=econ.dropna()
    econ_min1,econ_max1=get_df_period(econ2)    
    sec_min1,sec_max1=get_df_period(sec)
    min1=max(econ_min1,sec_min1)
    max1=min(econ_max1,sec_max1)
    econ3=set_df_period(econ,min1,max1)    
    sec3=set_df_period(sec,min1,max1)    
    df1=econ3; df2=sec3
    
    colname1=econ_growth
    #label1=econ_growth
    label1="增长率%"
    titletxt=ectranslate(scope)+": 经济增长与证券市场的关系"
    plot_line2(df1,ectranslate(ticker1),colname1,label1, \
               df2,ectranslate(ticker2),colname2,label2, \
               ylabeltxt,titletxt,footnote, \
               power=0,twinx=True)
    return econ,sec

if __name__=='__main__':
    fromdate='2010-1-1'; todate='2019-12-31'
    scope='China'; econ_factor='GDP'; sec_ticker='000001.SS'

    df_cn1=economy_security('China','1995-1-1','2019-12-31','GDP Per Capita','000001.SS')    
    df_cn2=economy_security('China','1995-1-1','2019-12-31','CNP GDP','000001.SS')
    df_us1=economy_security('USA','1980-1-1','2019-12-31','GDP','^DJI')
    df_us2=economy_security('USA','1980-1-1','2019-12-31','GDP','^GSPC')
#==============================================================================
def get_df_period(df):
    """
    功能： 获得df中日期索引的最小最大值
    """
    df_min=min(df.index)
    df_max=max(df.index)
    return df_min,df_max

if __name__=='__main__':
    import siat.security_prices as ssp
    df=ssp.get_price('AAPL','2020-1-1','2020-1-31')
    get_df_period(df)
    

#==============================================================================
#==============================================================================
#==============================================================================

def fred_factor_codes():
   import pandas as pd
   s=pd.DataFrame([
        # GDP未扣除通胀和汇率变化因素
        ['China','GDP','Annual','Gross Domestic Product','MKTGDPCNA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['USA','GDP','Annual','Gross Domestic Product','MKTGDPUSA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Japan','GDP','Annual','Gross Domestic Product','MKTGDPJPA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['India','GDP','Annual','Gross Domestic Product','MKTGDPINA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Korea','GDP','Annual','Gross Domestic Product','MKTGDPKRA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Russia','GDP','Annual','Gross Domestic Product','MKTGDPRUA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Singapore','GDP','Annual','Gross Domestic Product','MKTGDPSGA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Malaysia','GDP','Annual','Gross Domestic Product','MKTGDPMYA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Indonesia','GDP','Annual','Gross Domestic Product','MKTGDPIDA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Vietnam','GDP','Annual','Gross Domestic Product','MKTGDPVNA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Thailand','GDP','Annual','Gross Domestic Product','MKTGDPTHA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Australia','GDP','Annual','Gross Domestic Product','MKTGDPAUA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['France','GDP','Annual','Gross Domestic Product','MKTGDPFRA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Germany','GDP','Annual','Gross Domestic Product','MKTGDPDEA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['UK','GDP','Annual','Gross Domestic Product','MKTGDPGBA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Israel','GDP','Annual','Gross Domestic Product','MKTGDPILA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Italy','GDP','Annual','Gross Domestic Product','MKTGDPITA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Cambodia','GDP','Annual','Gross Domestic Product','MKTGDPKHA646NWDB','Current USD, Not Seasonally Adjusted'],
        
        #真实GDP，不变价格，年度，2011美元计价
        ['India','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAINA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['China','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNACNA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Japan','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAJPA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Korea','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAKRA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Thailand','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNATHA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Vietnam','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAVNA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Cambodia','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAKHA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Hong Kong','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAHKA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Singapore','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNASGA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Malaysia','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAMYA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Indonesia','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAIDA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['USA','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAUSA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['UK','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAGBA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['France','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAFRA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Germany','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNADEA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Italy','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAITA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Israel','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAILA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Russia','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNARUA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        ['Australia','CNP GDP','Annual','Real GDP at Constant National Prices','RGDPNAAUA666NRUG','2011 USD Millions, Not Seasonally Adjusted'],
        
        # 真实GDP：扣除通胀和与美元汇率变化因素，本地货币，季度
        # Real GDP is an inflation-adjusted measure, also as "constant price"/"inflation-cirrected"/"constant dollar"
        ['China','Constant GDP','Quarterly','Current Price Gross Domestic Product','CHNGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['Japan','Constant GDP','Quarterly','Current Price Gross Domestic Product','JPNGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['USA','Constant GDP','Quarterly','Current Price Gross Domestic Product','USAGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['Korea','Constant GDP','Quarterly','Current Price Gross Domestic Product','KORGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['Russia','Constant GDP','Quarterly','Current Price Gross Domestic Product','RUSGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['India','Constant GDP','Quarterly','Current Price Gross Domestic Product','INDGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['France','Constant GDP','Quarterly','Current Price Gross Domestic Product','FRAGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['Germany','Constant GDP','Quarterly','Current Price Gross Domestic Product','DEUGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['UK','Constant GDP','Quarterly','Current Price Gross Domestic Product','GBRGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        ['Australia','Constant GDP','Quarterly','Current Price Gross Domestic Product','AUSGDPNQDSMEI','Local Currency, Seasonally Adjusted'],
        
        # Ratio of GNP to GDP
        #['China','GNP','Annual','Ratio of GNP to GDP','GNPGDPCNA156NUPN','Percent, Not Seasonally Adjusted'],
        #['USA','GNP','Annual','Ratio of GNP to GDP','GNPGDPUSA156NUPN','Percent, Not Seasonally Adjusted'],
        #['Japan','GNP','Annual','Ratio of GNP to GDP','GNPGDPJPA156NUPN','Percent, Not Seasonally Adjusted'],

        ['China','GNP Ratio','Annual','Ratio of GNP to GDP','GNPGDPCNA156NUPN','Percent, Not Seasonally Adjusted'],
        ['USA','GNP Ratio','Annual','Ratio of GNP to GDP','GNPGDPUSA156NUPN','Percent, Not Seasonally Adjusted'],
        ['Japan','GNP Ratio','Annual','Ratio of GNP to GDP','GNPGDPJPA156NUPN','Percent, Not Seasonally Adjusted'],

        
        # 人均GDP：未扣除通胀因素
        ['China','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPCNA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Japan','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPJPA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['USA','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPUSA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Korea','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPKRA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['India','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPINA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Singapore','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPSGA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Malaysia','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPMYA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Indonesia','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPIDA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Vietnam','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPVNA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Thailand','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPTHA646NWDB','Current USD, Not Seasonally Adjusted'],
        ['Cambodia','GDP Per Capita','Annual','Gross Domestic Product Per Capita','PCAGDPKHA646NWDB','Current USD, Not Seasonally Adjusted'],
        
        # 不变价格人均GDP：扣除通胀和汇率因素
        ['China','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDCHN','2010 USD, Not Seasonally Adjusted'],
        ['USA','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDUSA','2010 USD, Not Seasonally Adjusted'],
        ['Japan','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDJPN','2010 USD, Not Seasonally Adjusted'],
        ['Russia','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDRUS','2010 USD, Not Seasonally Adjusted'],
        ['France','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDFRA','2010 USD, Not Seasonally Adjusted'],
        ['Germany','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDDEU','2010 USD, Not Seasonally Adjusted'],
        ['Italy','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDITA','2010 USD, Not Seasonally Adjusted'],
        ['Singapore','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDSGP','2010 USD, Not Seasonally Adjusted'],
        ['Malaysia','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDMYS','2010 USD, Not Seasonally Adjusted'],
        ['Indonesia','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDIDN','2010 USD, Not Seasonally Adjusted'],
        ['India','CNP GDP Per Capita','Annual','Constant Price GDP Per Capita','NYGDPPCAPKDIND','2010 USD, Not Seasonally Adjusted'],
        
        # CPI
        ['China','Constant CPI','Monthly','Consumer Price Index: All Items','CHNCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['USA','Constant CPI','Monthly','Consumer Price Index: All Items','USACPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Japan','Constant CPI','Monthly','Consumer Price Index: All Items','JPNCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Korea','Constant CPI','Monthly','Consumer Price Index: All Items','KORCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['India','Constant CPI','Monthly','Consumer Price Index: All Items','INDCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['France','Constant CPI','Monthly','Consumer Price Index: All Items','FRACPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Germany','Constant CPI','Monthly','Consumer Price Index: All Items','DEUCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Australia','Constant CPI','Monthly','Consumer Price Index: All Items','AUSCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['UK','CPI','Constant Monthly','Consumer Price Index: All Items','GBRCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Italy','Constant CPI','Monthly','Consumer Price Index: All Items','ITACPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Spain','Constant CPI','Monthly','Consumer Price Index: All Items','ESPCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Russia','Constant CPI','Monthly','Consumer Price Index: All Items','RUSCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Indonesia','Constant CPI','Monthly','Consumer Price Index: All Items','IDNCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],
        ['Israel','Constant CPI','Monthly','Consumer Price Index: All Items','ISRCPIALLMINMEI','Index 2015=100, Not Seasonally Adjusted'],

        # CPI%, Growth Rate over Previous Period
        ['USA','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01USM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['China','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01CNM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['Japan','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01USM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['Korea','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01KRM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['Russia','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01RUM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['India','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01INM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['France','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01FRM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['Germany','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01DEM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['Canada','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01CAM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],
        ['UK','MoM CPI','Monthly','Consumer Price Index: All Items Growth Rate','CPALTT01GBM657N','Growth Rate Previous Period, Not Seasonally Adjusted'],

        # PPI
        ['Sweden','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01SEM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Spain','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01ESM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['UK','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01GBM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Italy','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01ITM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Euro Area','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01EZM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Switzerland','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01CHM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['France','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01FRM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Germany','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI01DEM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['USA','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEAMP01USM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Mexico','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI02MXM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Japan','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI02JPM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Korea','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI02KRM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Russia','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEATI02RUM661N','Index 2015=100, Not Seasonally Adjusted'],
        ['Canada','Constant PPI','Monthly','Producer Prices Index: Total Industrial Activities','PIEAMP01CAM661N','Index 2015=100, Not Seasonally Adjusted'],
        
        # PPI China, up to 2015 & Annualonly
        ['China','Constant PPI','Annual','Producer Prices Index: Total Industrial Activities','PIEATI01CNA661N','Index 2015=100, Not Seasonally Adjusted'],

        # PPI%, Growth Rate same period previous year
        ['China','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','CHNPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['UK','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','GBRPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['France','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','FRAPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['Germany','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','DEUPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['Japan','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','JPNPIEATI02GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['Korea','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','KORPIEATI02GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['Italy','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','ITAPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['Spain','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','ITAPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['Poland','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','POLPIEATI01GYM','Growth rate same period previous year, Not Seasonally Adjusted'],
        ['USA','YoY PPI','Monthly','Producer Prices Index: Industrial Activities','PIEAMP02USM659N','Growth rate same period previous year, Not Seasonally Adjusted'],
        
        # Interest rate, Discount rate，中央银行的再贴现率，央行给予商业银行的短期贷款利率
        ['China','Discount Rate','Monthly','Central Bank Discount Rate','INTDSRCNM193N','Percent per Annum, Not Seasonally Adjusted'],
        ['USA','Discount Rate','Monthly','Central Bank Discount Rate','INTDSRUSM193N','Percent per Annum, Not Seasonally Adjusted'],
        ['India','Discount Rate','Monthly','Central Bank Discount Rate','INTDSRINM193N','Percent per Annum, Not Seasonally Adjusted'],
        ['Japan','Discount Rate','Monthly','Central Bank Discount Rate','INTDSRJPM193N','Percent per Annum, Not Seasonally Adjusted'],
        ['Korea','Discount Rate','Monthly','Central Bank Discount Rate','INTDSRKRM193N','Percent per Annum, Not Seasonally Adjusted'],
        ['Brazil','Discount Rate','Monthly','Central Bank Discount Rate','INTDSRBRM193N','Percent per Annum, Not Seasonally Adjusted'],
        
        # Immediate Interest rate, less than 24 hours, interbank rate
        ['China','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01CNM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['USA','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01USM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['Japan','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01JPM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['Korea','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01KRM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['India','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01INM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['France','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01FRM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['Germany','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01DEM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['Australia','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01AUM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['Indonesia','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01IDM156N','Percent per Annum, Not Seasonally Adjusted'],
        ['Russia','Immediate Rate','Monthly','Immediate Rates: Less than 24 Hours: Interbank Rate','IRSTCI01RUM156N','Percent per Annum, Not Seasonally Adjusted'],

        ['LIBOR','Immediate Rate','Daily','Overnight London Interbank Offered Rate, based on USD','USDONTD156N','Percent per Annum, Not Seasonally Adjusted'],
        ['UK','LIBOR','Daily','Overnight London Interbank Offered Rate, based on USD','USDONTD156N','Percent per Annum, Not Seasonally Adjusted'],
        
        # Foreign exchange rate, to USD
        ['China','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXCHUS','Chinese Yuan/1 US$, Not Seasonally Adjusted'],
        ['Japan','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXJPUS','Japanese Yen/1 US$, Not Seasonally Adjusted'],
        ['Korea','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXKOUS','South Korea Won/1 US$, Not Seasonally Adjusted'],
        ['Singapore','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXSIUS','Singapore Dollars/1 US$, Not Seasonally Adjusted'],
        ['China Hong Kong','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXHKUS','HK Dollars/1 US$, Not Seasonally Adjusted'],
        ['Australia','Exchange Rate','Daily','USD/Local Currency Foreign Exchange Rate','DEXUSAL','US Dollars/1 Australian Dollar, Not Seasonally Adjusted'],
        ['Euro','Exchange Rate','Daily','USD/Local Currency Foreign Exchange Rate','DEXUSEU','US Dollars/1 Euro, Not Seasonally Adjusted'],
        ['India','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXINUS','Indian Rupees/1 US$, Not Seasonally Adjusted'],
        ['UK','Exchange Rate','Daily','USD/Local Currency Foreign Exchange Rate','DEXUSUK','US Dollars/1 British Pound, Not Seasonally Adjusted'],
        ['Canada','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXCAUS','Canadian Dollars/1 US$, Not Seasonally Adjusted'],
        ['Mexico','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXMXUS','Mexican New Pesos/1 US$, Not Seasonally Adjusted'],
        ['Brazil','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXBZUS','Brazilian Reals/1 US$, Not Seasonally Adjusted'],
        ['Venezuela','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXVZUS','Venezuelan Bolivares/1 US$, Not Seasonally Adjusted'],
        ['South Africa','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXSFUS','South African Rand/1 US$, Not Seasonally Adjusted'],
        ['Sweden','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXSDUS','Swedish Kronor/1 US$, Not Seasonally Adjusted'],
        ['Thailand','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXTHUS','Thai Baht/1 US$, Not Seasonally Adjusted'],
        ['New Zealand','Exchange Rate','Daily','USD/Local Currency Foreign Exchange Rate','DEXUSNZ','US Dollars/1 New Zealand Dollar, Not Seasonally Adjusted'],
        ['China Taiwan','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXTAUS','New Taiwan Dollars/1 US$, Not Seasonally Adjusted'],
        ['Malaysia','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXMAUS','Malaysian Ringgit/1 US$, Not Seasonally Adjusted'],
        ['Denmark','Exchange Rate','Daily','Local Currency/USD Foreign Exchange Rate','DEXDNUS','Danish Kroner/1 US$, Not Seasonally Adjusted'],
        
        # M0，本地货币，月度
        ['China','M0','Monthly','National Monetary Supply M0','MYAGM0CNM189N','National Currency, Seasonally Adjusted'],
        
        # M1，本地货币，月度
        ['China','M1','Monthly','National Monetary Supply M1','MANMM101CNM189S','National Currency, Seasonally Adjusted'],
        ['USA','M1','Monthly','National Monetary Supply M1','MANMM101USM189S','National Currency, Seasonally Adjusted'],
        ['Japan','M1','Monthly','National Monetary Supply M1','MANMM101JPM189S','National Currency, Seasonally Adjusted'],
        ['Euro Area','M1','Monthly','Euro Area Monetary Supply M1','MANMM101EZM189S','National Currency, Seasonally Adjusted'],
        ['Korea','M1','Monthly','National Monetary Supply M1','MANMM101KRM189S','National Currency, Seasonally Adjusted'],
        ['India','M1','Monthly','National Monetary Supply M1','MANMM101INM189S','National Currency, Seasonally Adjusted'],
        
        # M2，本地货币，月度
        ['China','M2','Monthly','National Monetary Supply M2','MYAGM2CNM189N','National Currency, Not Seasonally Adjusted'],
        ['Japan','M2','Monthly','National Monetary Supply M2','MYAGM2JPM189N','National Currency, Seasonally Adjusted'],
        ['Korea','M2','Monthly','National Monetary Supply M2','MYAGM2KRM189N','National Currency, Not Seasonally Adjusted'],
        ['France','M2','Monthly','National Monetary Supply M2','MYAGM2FRM189N','National Currency, Not Seasonally Adjusted'],
        ['Italy','M2','Monthly','National Monetary Supply M2','MYAGM2ITM189N','National Currency, Not Seasonally Adjusted'],
        ['Russia','M2','Monthly','National Monetary Supply M2','MYAGM2RUM189N','National Currency, Not Seasonally Adjusted'],
        ['Indonesia','M2','Monthly','National Monetary Supply M2','MYAGM2IDM189N','National Currency, Not Seasonally Adjusted'],
        ['Brazil','M2','Monthly','National Monetary Supply M2','MYAGM2BRM189N','National Currency, Not Seasonally Adjusted'],
        ['USA','M2','Monthly','National Monetary Supply M2','M2NS','Billions of Dollars, Not Seasonally Adjusted'],

        # M3，本地货币，月度
        ['China','M3','Monthly','National Monetary Supply M3','MABMM301CNM189S','National Currency, Seasonally Adjusted'],
        ['USA','M3','Monthly','National Monetary Supply M3','MABMM301USM189S','National Currency, Seasonally Adjusted'],
        ['Japan','M3','Monthly','National Monetary Supply M3','MABMM301JPM189S','National Currency, Seasonally Adjusted'],
        ['UK','M3','Monthly','National Monetary Supply M3','MABMM301GBM189S','National Currency, Seasonally Adjusted'],
        ['Euro Area','M3','Monthly','Euro Area Monetary Supply M3','MABMM301EZM189S','National Currency, Seasonally Adjusted'],
        ['Australia','M3','Monthly','National Monetary Supply M3','MABMM301AUM189S','National Currency, Seasonally Adjusted'],

        # Stock Market Cap to GDP
        ['China','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01CNA156NWDB','Percent, Not Seasonally Adjusted'],
        ['USA','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01USA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Japan','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01JPA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Korea','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01KRA156NWDB','Percent, Not Seasonally Adjusted'],
        ['India','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01INA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Singapore','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01SGA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Malaysia','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01MYA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Indonesia','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01IDA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Vietnam','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01VNA156NWDB','Percent, Not Seasonally Adjusted'],
        ['China Hong Kong','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01HKA156NWDB','Percent, Not Seasonally Adjusted'],
        ['France','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01FRA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Germany','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01DEA156NWDB','Percent, Not Seasonally Adjusted'],
        ['UK','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01GBA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Italy','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01ITA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Spain','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01ESA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Israel','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01ILA156NWDB','Percent, Not Seasonally Adjusted'],
        ['Canada','SMC to GDP','Annual','Stock Market Capitalization to GDP','DDDM01CAA156NWDB','Percent, Not Seasonally Adjusted'],

        # 消费者信心综合指数：OECD
        ['China','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03CNM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['USA','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03USM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['Japan','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03JPM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['Korea','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03KRM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['Indonesia','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03IDM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['France','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03FRM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['Germany','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03DEM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['Italy','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03ITM665S','Normalised (Normal=100), Seasonally Adjusted'],
        ['Spain','Consumer Confidence','Monthly','Consumer Confidence Composite Indicator by OECD','CSCICP03ESM665S','Normalised (Normal=100), Seasonally Adjusted'],

        # 粗出生率；又称为出生率。指1年内平均每千人中出生人数的比率。它反映一定时期内人口的出生水平。
        ['China','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINCHN','Births per 1,000 People, Not Seasonally Adjusted'],
        ['USA','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINUSA','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Japan','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINJPN','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Korea','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINKOR','Births per 1,000 People, Not Seasonally Adjusted'],
        ['France','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINFRA','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Germany','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINDEU','Births per 1,000 People, Not Seasonally Adjusted'],
        ['India','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTININD','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Indonesia','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINIDN','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Singapore','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINSGP','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Vietnam','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINVNM','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Pakistan','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINPAK','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Cambodia','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINKHM','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Malaysia','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINMYS','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Australia','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINAUS','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Italy','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINITA','Births per 1,000 People, Not Seasonally Adjusted'],
        ['Spain','Birth Rate','Annual','Crude Birth Rate','SPDYNCBRTINESP','Births per 1,000 People, Not Seasonally Adjusted'],

        # 人口增长率；出生率-死亡率。
        #Annual population growth rate for year t is the exponential rate of growth of midyear population from year t-1 to t, 
        #expressed as a percentage.
        ['China','Population Growth','Annual','Population Growth by WB','SPPOPGROWCHN','Percent Change at Annual Rate, NSA'],
        ['USA','Population Growth','Annual','Population Growth by WB','SPPOPGROWUSA','Percent Change at Annual Rate, NSA'],
        ['Japan','Population Growth','Annual','Population Growth by WB','SPPOPGROWJPN','Percent Change at Annual Rate, NSA'],
        ['Korea','Population Growth','Annual','Population Growth by WB','SPPOPGROWKOR','Percent Change at Annual Rate, NSA'],
        ['India','Population Growth','Annual','Population Growth by WB','SPPOPGROWIND','Percent Change at Annual Rate, NSA'],

        # 青年人失业率：WB
        ['China','Youth Unemployment','Annual','Youth Unemployment Rate by WB','SLUEM1524ZSCHN','Percent(Age 15-24, seeking employment), NSA'],
        ['USA','Youth Unemployment','Annual','Youth Unemployment Rate by WB','SLUEM1524ZSUSA','Percent(Age 15-24, seeking employment), NSA'],
        ['Japan','Youth Unemployment','Annual','Youth Unemployment Rate by WB','SLUEM1524ZSJPN','Percent(Age 15-24, seeking employment), NSA'],
        ['Korea','Youth Unemployment','Annual','Youth Unemployment Rate by WB','SLUEM1524ZSKOR','Percent(Age 15-24, seeking employment), NSA'],
        ['India','Youth Unemployment','Annual','Youth Unemployment Rate by WB','SLUEM1524ZSIND','Percent(Age 15-24, seeking employment), NSA'],

        # GNI: Gross National Income
        ['China','GNI','Annual','Gross National Income','MKTGNICNA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['USA','GNI','Annual','Gross National Income','MKTGNIUSA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Japan','GNI','Annual','Gross National Income','MKTGNIJPA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Korea','GNI','Annual','Gross National Income','MKTGNIKRA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['India','GNI','Annual','Gross National Income','MKTGNIINA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Vietnam','GNI','Annual','Gross National Income','MKTGNIVNA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Thailand','GNI','Annual','Gross National Income','MKTGNITHA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Cambodia','GNI','Annual','Gross National Income','MKTGNIKHA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['China Hong Kong','GNI','Annual','Gross National Income','MKTGNIHKA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Malaysia','GNI','Annual','Gross National Income','MKTGNIMYA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Singapore','GNI','Annual','Gross National Income','MKTGNISGA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Indonesia','GNI','Annual','Gross National Income','MKTGNIIDA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Australia','GNI','Annual','Gross National Income','MKTGNIAUA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['New Zealand','GNI','Annual','Gross National Income','MKTGNINZA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['UK','GNI','Annual','Gross National Income','MKTGNIGBA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Germany','GNI','Annual','Gross National Income','MKTGNIDEA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['France','GNI','Annual','Gross National Income','MKTGNIFRA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],
        ['Spain','GNI','Annual','Gross National Income','MKTGNIESA646NWDB','Current U.S. Dollars, Not Seasonally Adjusted'],

        # 出口对进口的比例
        ['China','Exports to Imports','Monthly','Ratio of Exports to Imports','XTEITT01CNM156S','Percent, Seasonally Adjusted'],
        ['India','Exports to Imports','Monthly','Ratio of Exports to Imports','XTEITT01INM156N','Percent, Seasonally Adjusted'],
        ['Brazil','Exports to Imports','Monthly','Ratio of Exports to Imports','XTEITT01BRM156S','Percent, Seasonally Adjusted'],
        ['Russia','Exports to Imports','Monthly','Ratio of Exports to Imports','XTEITT01RUM156S','Percent, Seasonally Adjusted'],
        ['Indonesia','Exports to Imports','Monthly','Ratio of Exports to Imports','XTEITT01IDM156S','Percent, Seasonally Adjusted'],

        # 人口雇佣比例
        ['China','Employment to Population','Annual','Employment to Population Ratio by WB','SLEMPTOTLSPZSCHN','Percent, Not Seasonally Adjusted'],
        ['USA','Employment to Population','Annual','Employment to Population Ratio by WB','SLEMPTOTLSPZSUSA','Percent, Not Seasonally Adjusted'],
        ['Japan','Employment to Population','Annual','Employment to Population Ratio by WB','SLEMPTOTLSPZSJPN','Percent, Not Seasonally Adjusted'],
        ['Korea','Employment to Population','Annual','Employment to Population Ratio by WB','SLEMPTOTLSPZSKOR','Percent, Not Seasonally Adjusted'],
        ['India','Employment to Population','Annual','Employment to Population Ratio by WB','SLEMPTOTLSPZSIND','Percent, Not Seasonally Adjusted'],

        # 人口总数
        ['China','Population','Annual','Total Population','POPTOTCNA647NWDB','Persons, Not Seasonally Adjusted'],
        ['USA','Population','Annual','Total Population','POPTOTUSA647NWDB','Persons, Not Seasonally Adjusted'],
        ['Japan','Population','Annual','Total Population','POPTOTJPA647NWDB','Persons, Not Seasonally Adjusted'],
        ['Korea','Population','Annual','Total Population','POPTOTKRA647NWDB','Persons, Not Seasonally Adjusted'],
        ['India','Population','Annual','Total Population','POPTOTINA647NWDB','Persons, Not Seasonally Adjusted'],
        
        # 其他
        ], columns=['scope','factor','freq','name','symbol','units'])
    
   return s


#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
if __name__ =="__main__":
    fromdate='2020-1-5'
    todate='2020-10-1'

def pmi_china(fromdate,todate):
    """
    功能：绘制中国的PMI指数制造业/非制造业单线图
    """
    #检查日期期间的合理性
    valid,start,end=check_period(fromdate,todate)
    if not valid:
        print('  Error(pmi_china): period not valid:',fromdate,todate)
        return None      
    
    #日期变换，将日都重置为每月的第一日
    year=start.year
    month=start.month    
    fromdate1=str(year)+'-'+str(month)+'-'+'1'

    year=end.year
    month=end.month    
    todate1=str(year)+'-'+str(month)+'-'+'1'
    _,start,end=check_period(fromdate1,todate1)
    
    #获取PMI数据
    import akshare as ak
    df = ak.macro_china_pmi()
    
    #截取日期区间数据
    import pandas as pd
    df['date']=pd.to_datetime(df['月份'])
    df.set_index('date',inplace=True)    
    df1=df[df.index >= start]
    df2=df1[df1.index <= end]
    
    #形成水平50线
    df2['benchmark']=50   
    
    #将字符串表示的数值转换为数值型，否则绘图很乱
    df2['制造业PMI']=df2['制造业-指数'].astype('float')
    df2['非制造业PMI']=df2['非制造业-指数'].astype('float')
    
    #绘图：制造业
    ticker1=ticker2='PMI'
    colname2="benchmark"
    label2="景气/衰退分界线"
    ylabeltxt=''
    
    import datetime
    today=datetime.date.today()
    footnote="数据来源：东方财富，"+str(today)
    
    colname1="制造业PMI"
    label1="制造业"    
    titletxt="中国采购经理人指数PMI：制造业"
    plot_line2(df2,ticker1,colname1,label1, \
               df2,ticker2,colname2,label2, \
               ylabeltxt,titletxt,footnote)
    
    #绘图：非制造业
    colname1="非制造业PMI"
    label1="非制造业"    
    titletxt="中国采购经理人指数PMI：非制造业"
    plot_line2(df2,ticker1,colname1,label1, \
               df2,ticker2,colname2,label2, \
               ylabeltxt,titletxt,footnote)    
    #返回数据
    return df2

#==============================================================================
#==============================================================================
#==============================================================================
if __name__ =="__main__":
    fieldlist=['VALUE','scope','factor','freq','name','units']

def internal_growth_rate_df(df0,fieldlist=['VALUE','scope','factor','freq','name','units']):
    """
    功能：计算内部增长率IRR
    rdf的结构：VALUE为具体的数值，scope为国家或地区，factor为指标，freq为频度。
    """
    df=df0.copy()
    df.dropna(inplace=True)
    
    fld_value=fieldlist[0]
    fld_scope=fieldlist[1]
    scope=df[fld_scope][0]
    
    fld_factor=fieldlist[2]
    factor=df[fld_factor][0]
    
    fld_freq=fieldlist[3]
    freq=df[fld_freq][0]
    
    fld_name=fieldlist[4]
    fld_units=fieldlist[5]
    
    #开始值和结束值
    value0=df[fld_value][0]
    value1=df[fld_value][-1]
    num=len(df)
    item_gr=pow(value1/value0,1/num)-1.0
    
    annual_gr=item_gr
    if freq == 'Daily':
        annual_gr=pow(1+item_gr,365)-1.0
    if freq == 'Monthly':
        annual_gr=pow(1+item_gr,12)-1.0
    if freq == 'Quarterly':
        annual_gr=pow(1+item_gr,4)-1.0
    
    #计算期间
    date0=str(df.index[0].year)+'年'
    date1=str(df.index[-1].year)+'年'
    
    if freq in ['Monthly','Daily']:
        date0=date0+str(df.index[0].month)+'月'
        date1=date1+str(df.index[-1].month)+'月'        
    
    if freq in ['Daily']:
        date0=date0+str(df.index[0].day)+'日'
        date1=date1+str(df.index[-1].day)+'日'     
    
    #显示增长率
    #print('\n')
    print('从'+date0+'至'+date1+'：')
    if freq != 'Annual':
        print('    '+ectranslate(scope)+ectranslate(factor)+'的'+ectranslate(freq)+'均环比增长率：',round(item_gr*100.0,4),'\b%')
    print('    '+ectranslate(scope)+ectranslate(factor)+'的年均增长率：',round(annual_gr*100.0,4),'\b%')
    return 

if __name__ =="__main__":
    internal_growth_rate_df(df)
    

def internal_growth_rate(rvar,fieldlist=['VALUE','scope','factor','freq','name','units']):
    """
    功能：计算内部增长率IRR
    rvar：可能为df或dict。若为dict，里面的元素为两个df。
    每个df的结构：VALUE为具体的数值，scope为国家或地区，factor为指标，freq为频度。
    """
    print('')
    import pandas as pd
    if isinstance(rvar,pd.DataFrame):
        internal_growth_rate_df(rvar,fieldlist=['VALUE','scope','factor','freq','name','units'])
        return
    
    if isinstance(rvar,tuple):
        for k in rvar:
            internal_growth_rate_df(k,fieldlist=['VALUE','scope','factor','freq','name','units'])
        return
    
    return
        
if __name__ =="__main__":
    internal_growth_rate(df)
#==============================================================================
#==============================================================================
#==============================================================================

    


# -*- coding: utf-8 -*-
"""
本模块功能：CAPM模型贝塔系数计算
所属工具包：证券投资分析工具SIAT 
SIAT：Security Investment Analysis Tool
创建日期：2018年7月16日
最新修订日期：2020年1月15日
作者：王德宏 (WANG Dehong, Peter)
作者单位：北京外国语大学国际商学院
作者邮件：wdehong2000@163.com
版权所有：王德宏
用途限制：仅限研究与教学使用，不可商用！商用需要额外授权。
特别声明：作者不对使用本工具进行证券投资导致的任何损益负责！
"""

#==============================================================================
import warnings; warnings.filterwarnings('ignore')
from siat.common import *
from siat.translate import *
from siat.security_prices import *
#==============================================================================
import matplotlib.pyplot as plt

#处理绘图汉字乱码问题
import sys; czxt=sys.platform
if czxt in ['win32','win64']:
    plt.rcParams['font.sans-serif'] = ['SimHei']  # 设置默认字体
    mpfrc={'font.family': 'SimHei'}

if czxt in ['darwin']: #MacOSX
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family': 'Heiti TC'}

if czxt in ['linux']: #website Jupyter
    plt.rcParams['font.family']= ['Heiti TC']
    mpfrc={'font.family':'Heiti TC'}

# 解决保存图像时'-'显示为方块的问题
plt.rcParams['axes.unicode_minus'] = False 
#==============================================================================
def get_price_excel(ticker,fromdate,todate,excelfile,sheetname='Sheet1'):
    """
    功能：抓取股价，保存到Excel文件中
    输出：指定收盘价格序列，最新日期的股价排列在前
    ticker: 股票代码。大陆股票代码加上后缀.SZ或.SS，港股代码去掉前导0加后缀.HK
    fromdate: 样本开始日期，尽量远的日期，以便取得足够多的原始样本
    todate: 样本结束日期，既可以是今天日期，也可以是一个历史日期
    excelfile：保存到Excel文件名，含目录。如果目录不存在，将会导致错误。
    """
    
    #仅为调试用的函数入口参数，正式使用前需要注释掉！
    #ticker='000300.SS'
    #fromdate='01/01/2018'
    #todate='06/30/2019'
    #---------------------------------------------
    
    #仅为测试使用
    #ticker='601857.SS'
    #fromdate='2019-1-1'
    #todate='2019-8-31'
    #excelfile='C:/temp/myexcel1.xlsx'
    #sheetname='601857.SS'

    price=get_price(ticker,fromdate,todate)
    if price is None:
        print("  #Error(get_price_excel): failed to retrieve stock data")
        print("  Information:",ticker,fromdate,todate) 
        return
    try:
        price.to_excel(excelfile,sheetname,encoding='utf-8')             
    except:
        print("  #Error((get_price_excel): failed to stock data to excel file")        
        print("  Information:",excelfile,sheetname)
        return
    print("***Results saved in",excelfile,"with sheet",sheetname)
    return 
    

if __name__=='__main__':
    df601857=get_price_excel('601857.SS','2019-1-1','2019-12-31', \
                             'C:/temp/myexcel1.xlsx','601857.SS')


#==============================================================================
def prepare_capm_data(stkcd,mktidx,start,end):
    """
    函数功能：准备计算一只股票CAPM模型贝塔系数的数据，并标记年度
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    start：使用股票价格数据的开始日期，MM/DD/YYYY
    end：使用股票价格数据的结束日期，MM/DD/YYYY
    输出数据：
    返回数据：带年度标记的可直接用于capm回归的股票收益率数据
    """
        
    #仅用于调试，正式使用前应注释掉
    #stkcd='600028.SS'; mktidx='000001.SS'
    #start="12/31/2011"; end="12/31/2018"

    #抓取股价和指数
    import siat.security_prices as ssp
    stock=get_price(stkcd,start,end)
    if stock is None:
        print("  #Error(prepare_capm_data): failed to get stock price")
        print("  Information:",stkcd,start,end)              
        return None    
    market=get_price(mktidx,start,end)
    if market is None:
        print("  #Error(prepare_capm_data): failed to get market index")
        print("  Information:",mktidx,start,end)              
        return None

    #计算日收益率
    import pandas as pd
    stkret=pd.DataFrame(stock['Close'].pct_change())
    mktret=pd.DataFrame(market['Close'].pct_change())

    #合并，去掉空缺
    R=pd.merge(mktret,stkret,how='left',left_index=True,right_index=True)
    R=R.dropna()

    #标记各个年度
    R['Year']=R.index.strftime("%Y")

    #返回带年份的股票收益率序列
    return R

#==============================================================================
def capm_beta(stkcd,mktidx,start,end):
    """
    函数功能：计算一只股票的静态CAPM模型贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    start：使用股票价格数据的开始日期，MM/DD/YYYY
    end：使用股票价格数据的结束日期，MM/DD/YYYY
    输出数据：
    显示CAPM市场模型回归的alpha, beta, 以及显著性和拟合优度
    无返回数据
    """
        
    #读取股价并准备好收益率数据
    R=prepare_capm_data(stkcd,mktidx,start,end)
    if R is None:
        print("  #Error(capm_beta): failed to prepare capm data")
        print("  Info:",stkcd,mktidx,start,end)              
        return   

    #全数据OLS回归
    from scipy import stats 
    (beta,alpha,r_value,p_value,std_err)= \
        stats.linregress(R['Close_x'],R['Close_y'])

    #显示回归结果
    print(" \n===== 计算CAPM贝塔系数 =====")
    print("回归模型：市场模型")
    print(equalwidth("股票",16), codetranslate(stkcd))
    print(equalwidth("市场指数",16), codetranslate(mktidx))
    print(equalwidth("样本期间开始于",16), start)
    print(equalwidth("样本期间结束于",16), end)
    print(equalwidth("截距项",16), round(alpha,4))
    print(equalwidth("贝塔系数",16), round(beta,4))
    print(equalwidth("R-square",16), round(r_value**2,4))
    print(equalwidth("p-value",16), round(p_value,4))
    
    import datetime
    today = datetime.date.today()
    print("*数据来源: 雅虎财经,",today)

    return  beta  
    
if __name__=='__main__':
    capm_beta('002504.SZ','000001.SS','12/31/2011','12/31/2018')    

#==============================================================================

def plot_trend(titletxt,footnotetxt,df,power=1):
    """
    功能：绘制散点数据的平滑曲线图及其趋势线
    titletxt: 标题
    footnotetxt：脚注
    df: 数据框，其索引为字符型年份，第1列与年份对应的数值1，
    第2列与年份对应的数值2，。。。
    power: 趋势线拟合的幂阶数，默认为三阶多项式
    """
    #将字符型年份转化为整数型
    yearstr=df.index
    year=[]
    for t in yearstr:
        tt=int(t)
        year=year+[tt]    
    
    #取得列名
    rowlist=df.columns.values.tolist()
    n=len(rowlist)
    
    import numpy as np
    import scipy.interpolate as itp

    #由于年度为连续整数，为插值方便，先将所有年度放大10倍
    year0=[]
    for t in year:
        tt=t*10
        year0=year0+[tt]

    #形成用于插值的连续整数自变量xx
    x1=year0[0]
    x2=year0[-1]+1
    xx=np.array(range(x1,x2,1))   
    
    #将用于插值的自变量缩小10倍，以便自变量在x轴上的尺度统一
    x=[]
    for t in xx:
        tt=t/10.0
        x=x+[tt]
    
    #循环处理每个因变量
    for i in np.array(range(0,n,1)):
        number=df[rowlist[i]]

        #进行插值: linear=线性插值, cubic=立方插值
        y=itp.interp1d(year0,number,kind='cubic')
        #生成插值出来的模拟数据因变量
        y2=y(xx)
        #绘图1：插值出来的大量模拟点构成的平滑曲线
        plt.plot(x,y2)
        #绘图2：原数据点

        #进行拟合: 1=线性拟合, 2=二次拟合，3=三次拟合
        p=np.polyfit(year0,number,power)
        #生成拟合出来的模拟数据因变量
        p2=np.polyval(p,xx)
        #绘图2：拟合出来的大量模拟点构成的平滑趋势曲线
        plt.plot(x,p2,linestyle=':',linewidth=3)

        #绘图3：原数据点
        plt.plot(year,number,'*',label=rowlist[i])
    
    #x轴刻度：使用原数据点的自变量
    plt.xticks(year,rotation=45)
    plt.legend(loc='best')
    plt.title(titletxt)
    plt.xlabel(footnotetxt)
    plt.show()
    
    return
#==============================================================================

def capm_beta_yearly(stkcd,mktidx,yearlist):
    """
    函数功能：分年度计算一只股票的CAPM模型贝塔系数
    输入参数：
    stkcd: 股票代码
    mktidx: 指数代码
    yearlist: 年度列表，start日期要给第一个年份留出至少一年数据供回归使用
    输出数据：
    按年份显示CAPM市场模型回归的alpha, beta, 以及显著性和拟合优度
    无返回数据
    """
    
    #运行开始信息
    print("*** Analysing yearly CAPM betas, please wait ...")
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31' 
        
    #读取股价并准备好收益率数据
    R=prepare_capm_data(stkcd,mktidx,start,end)
    if R is None:
        print("  #Error(capm_beta_yearly): failed to prepare capm data")
        print("  Information:",stkcd,mktidx,start,end)              
        return      

    #print("\n***CAPM yearly beta: Stock_return=alpha+beta*Market_return")
    #print("Stock code:",stkcd)
    #print("Market index:",mktidx)
    #print("Year Beta   R-sqr  p-value")

    #用于保存beta(CAPM)
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta','alpha','R-sqr','p-value'))
    
    #分年度OLS回归
    from scipy import stats
    for year in yearlist:
        #print(year,' ',end='')
        r=R[R['Year']==year]
        if len(r)==0: continue
        try:
            output=stats.linregress(r['Close_x'],r['Close_y'])
        except:
            print("  #Error(capm_beta_yearly): no data for regression")
            print("  Information:",stkcd,year,R,r)
            return             
        (beta,alpha,r_value,p_value,std_err)=output
        
        row=pd.Series({'Year':year,'Beta':beta,'alpha':alpha, \
                       'R-sqr':r_value**2,'p-value':p_value})
        betas=betas.append(row,ignore_index=True)          
        #print(year,round(beta,4),round(r_value**2,3),round(p_value,4))
    
    #设置打印标题与数据对齐
    pd.set_option('display.unicode.ambiguous_as_wide', True)
    pd.set_option('display.unicode.east_asian_width', True)
    pd.set_option('display.width', 180) # 设置打印宽度(**重要**)
    
    print("\n ===== 股票的年度贝塔系数:",codetranslate(stkcd),"=====")
    print(betas.to_string(index=False))
    import datetime as dt; today=dt.date.today()    
    print(" 数据来源：新浪财经/雅虎财经，基于"+codetranslate(mktidx)+'，'+str(today))
    
    betas.set_index('Year',inplace=True)
    
    #绘图：年度贝塔系数趋势
    df=pd.DataFrame(betas['Beta'])
    title="股票的年度贝塔系数趋势: "+codetranslate(stkcd)
    foot="\n数据来源：新浪财经/雅虎财经，基于"+codetranslate(mktidx)+'，'+str(today)
    plot_trend(title,foot,df,power=3)
    
    return betas   
    
if __name__=='__main__':
    yearlist=['2013','2014','2015','2016','2017','2018','2019']
    capm_beta_yearly('AAPL','^GSPC',yearlist)    

#==============================================================================
#新增投资组合贝塔系数部分
#==============================================================================
#==============================================================================
def prepare_capm_portfolio(tickerlist,sharelist,mktidx,start,end):
    """
    函数功能：准备计算股票投资组合CAPM模型贝塔系数的数据，并标记年度
    输入参数：
    tickerlist: 股票代码列表
    sharelist：持有份额列表
    mktidx: 市场指数代码
    start：使用股票价格数据的开始日期，'YYYY-MM-DD'
    end：使用股票价格数据的结束日期
    输出数据：
    返回数据：带年度标记的可直接用于capm模型回归的数据表
    """
        
    #仅用于调试，正式使用前应注释掉
    #tickerlist=['AAPL','MSFT']
    #sharelist=[2,1]
    #mktidx='^GSPC'
    #start="1/1/2019"; end="8/31/2019"

    #抓取投资组合股价和指数
    import siat.security_prices as ssp
    stock=get_price_portfolio(tickerlist,sharelist,start,end)
    if (stock is None) or (len(stock)==0):
        print("  #Error(prepare_capm_portfolio): failed to get stock prices")
        print("  Info:",tickerlist,start,end)              
        return None    
    market=get_price(mktidx,start,end)
    if (market is None) or (len(market)==0):
        print("  #Error(prepare_capm_portfolio): failed to get market index")
        print("  Info:",mktidx,start,end)              
        return None

    #计算日收益率
    import pandas as pd
    stkret=pd.DataFrame(stock['Close'].pct_change())
    mktret=pd.DataFrame(market['Close'].pct_change())

    #合并：mktret作为Close_x，stkret作为Close_y，去掉空缺
    R=pd.merge(mktret,stkret,how='left',left_index=True,right_index=True)
    R=R.dropna()

    #标记各个年度
    R['Year']=R.index.strftime("%Y")

    #返回带年份的股票收益率序列
    return R

#==============================================================================
def capm_beta_portfolio(tickerlist,sharelist,mktidx,start,end):
    """
    函数功能：计算投资组合的静态CAPM模型贝塔系数
    输入参数：
    tickerlist: 股票代码列表
    sharelist：持有份额列表
    mktidx: 指数代码
    start：使用股票价格数据的开始日期，MM/DD/YYYY
    end：使用股票价格数据的结束日期，MM/DD/YYYY
    输出数据：
    显示CAPM市场模型回归的截距, beta, 以及显著性和拟合优度
    无返回数据
    """
        
    #读取股价并准备好收益率数据
    R=prepare_capm_portfolio(tickerlist,sharelist,mktidx,start,end)
    if R is None:
        print("  #Error(capm_beta_portfolio): failed to prepare capm data")
        print("  Information:",tickerlist,sharelist,mktidx,start,end)              
        return   

    #全数据OLS回归
    from scipy import stats 
    (beta,alpha,r_value,p_value,std_err)= \
        stats.linregress(R['Close_x'],R['Close_y'])

    #显示回归结果
    print("\n *** Portfolio CAPM Beta Coefficient \
          \n Model: Stock_return=intercept + beta * Market_return")
    print(" Portfolio stocks:",tickerlist)
    print(" Stock portions  :",sharelist)
    print(" Market index    :",mktidx)
    print(" Intercept       :",round(alpha,4))
    print(" Portfolio beta  :",round(beta,4))
    print(" R-sqr       :",round(r_value**2,4))
    print(" p-value         :",round(p_value,4))

    return beta


#==============================================================================
def capm_beta_portfolio_yearly(tickerlist,sharelist,mktidx,yearlist):
    """
    函数功能：分年度计算投资组合的CAPM模型贝塔系数，并绘图
    输入参数：
    tickerlist: 投资组合中各个成分股的股票代码
    sharelist: 各个成分股的持股比例
    mktidx: 指数代码
    yearlist: 年度列表，start日期要给第一个年份留出至少一年数据供回归使用
    输出数据：
    按年份绘图beta
    无返回数据
    """
    
    #运行开始信息
    print("*** Analysing yearly portfolio CAPM betas, please wait ...")
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31' 
        
    #读取股价并准备好收益率数据
    R=prepare_capm_portfolio(tickerlist,sharelist,mktidx,start,end)
    if (R is None) or (len(R)==0):
        print("  #Error(capm_beta_portfolio_yearly): failed to prepare capm data")
        print("  Info:",tickerlist,sharelist,mktidx,start,end)              
        return      

    #用于保存beta(CAPM)和beta
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta','intercept','R-sqr','p-value'))
    
    #分年度OLS回归
    from scipy import stats
    for year in yearlist:
        #print(year,' ',end='')
        r=R[R['Year']==year]
        if len(r)==0: continue
        try:
            output=stats.linregress(r['Close_x'],r['Close_y'])
        except:
            print("  #Error(capm_beta_portfolio_yearly): no data for regression")
            print("  Information:",year,R,r)
            return         
        (beta,alpha,r_value,p_value,std_err)=output
        
        row=pd.Series({'Year':year,'Beta':beta,'intercept':alpha, \
                       'R-sqr':r_value**2,'p-value':p_value})
        betas=betas.append(row,ignore_index=True)          
        
    print("\n",betas)
    betas.set_index('Year',inplace=True)
    
    #绘图：年度贝塔系数变化
    df=pd.DataFrame(betas['Beta'])
    title="投资组合的年度贝塔系数"+ \
            "\n成分股: "+str(tickerlist)+"\n持仓权重: "+str(sharelist)
    import datetime; today = datetime.date.today()
    foot="数据来源: 新浪财经/雅虎财经，"+str(today)
    plot_trend(title,foot,df,power=2)
    
    return  betas  
    
if __name__=='__main__':
    yearlist=['2013','2014','2015','2016','2017','2018','2019']
    capm_beta_portfolio_yearly(['AAPL','MSFT'],[2,1],'^GSPC',yearlist)
    capm_beta_portfolio_yearly(['600028.SS','600036.SS'],[2,1],'000001.SS',yearlist)    

#==============================================================================
def capm_beta_portfolio_yearly_excel(tickerlist,sharelist,mktidx,yearlist, \
                                     excelfile,sheetname='Sheet1'):
    """
    函数功能：分年度计算投资组合的CAPM模型贝塔系数，绘图，结果保存到Excel
    输入参数：
    tickerlist: 投资组合中各个成分股的股票代码
    sharelist: 各个成分股的持股比例
    mktidx: 指数代码
    yearlist: 年度列表
    excelfile: 带目录的Excel文件名，如果目录不存在则出错
    sheetname：Excel文件中的sheet名
    输出：
    按年份绘图beta系数, 并保存到Excel文件
    无返回数据
    """
    
    #运行开始信息
    print("\n*** Analysing yearly portfolio CAPM betas, please wait ...")
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31' 
        
    #读取股价并准备好收益率数据
    R=prepare_capm_portfolio(tickerlist,sharelist,mktidx,start,end)
    if R is None:
        print("  #Error(capm_beta_portfolioyearly_excel): failed to prepare capm data")
        print("  Information:",tickerlist,sharelist,mktidx,start,end)              
        return      

    #用于保存beta(CAPM)和beta
    import pandas as pd
    betas=pd.DataFrame(columns=('Year','Beta','intercept','R-sqr','p-value'))
    
    #分年度OLS回归
    from scipy import stats
    for year in yearlist:
        #print(year,' ',end='')
        r=R[R['Year']==year]
        if len(r)==0: continue
        try:
            output=stats.linregress(r['Close_x'],r['Close_y'])
        except:
            print("  #Error(capm_beta_portfolio_yearly_excel): no data available for regressing beta")
            print("  Information:",year,R,r)
            return         
        (beta,alpha,r_value,p_value,std_err)=output
        
        row=pd.Series({'Year':year,'Beta':beta,'intercept':alpha, \
                       'R-sqr':r_value**2,'p-value':p_value})
        betas=betas.append(row,ignore_index=True)          
        
    betas2=betas.copy() #用于保存excel   
    betas2['Year']=betas2['Year'].astype('int')
    betas2['Portfolio']=str(tickerlist)
    betas2['Share Ratio']=str(sharelist)
    betas2['Market Index']=mktidx
    
    betas.set_index('Year',inplace=True)    #用于绘图
    
    #绘图：年度贝塔系数变化
    df=pd.DataFrame(betas['Beta'])
    title="Investment Portfolio's Annual CAPM Betas"+ \
            "\nPortfolio: "+str(tickerlist)+"\nComposition: "+str(sharelist)
    foot="\nSource: Sina Finance/Yahoo Finance"
    plot_trend(title,foot,df,power=2)
    
    """
    #为折线加数据标签
    for a,b in zip(betas.index,betas['Beta']):
        plt.text(a,b+0.02,str(round(b,2)),ha='center',va='bottom',fontsize=7)
    plt.show()
    """
    
    #保存到Excel   
    import pandas as pd
    try:
        file1=pd.ExcelFile(excelfile)
    except:
        #不存在excelfile文件，直接写入
        betas2.to_excel(excelfile,sheet_name=sheetname, \
                       header=True,encoding='utf-8')
        print("***Results saved in",excelfile,"@ sheet",sheetname)
        return
    else:
        #已存在excelfile文件，先将所有已有内容读出        
        dict=pd.read_excel(file1, None)
    file1.close()
    
    #获得所有sheet名字
    sheetlist=list(dict.keys())
    
    #检查新的sheet名字是否已存在
    try:
        pos=sheetlist.index(sheetname)
    except:
        #不存在重复
        dup=False
    else:
        #存在重复，合并内容
        dup=True
        #print(dict[sheetlist[pos]],betas2)
        df1=dict[sheetlist[pos]][['Year','Beta','intercept','R-sqr','p-value','Portfolio','Share Ratio','Market Index']]
        df=pd.concat([df1,betas2],axis=0,ignore_index=True)        
        dict[sheetlist[pos]]=df
    
    #将原有内容写回excelfile    
    result=pd.ExcelWriter(excelfile)
    for s in sheetlist:
        df1=dict[s][['Year','Beta','intercept','R-sqr','p-value','Portfolio','Share Ratio','Market Index']]
        df1.to_excel(result,s,header=True,index=True,encoding='utf-8')
    #写入新内容
    if not dup: #sheetname未重复
        betas2.to_excel(result,sheetname,header=True,index=True,encoding='utf-8')
    try:
        result.save()
        result.close()
    except:
        print("  #Error(capm_beta_portfolio_yearly_excel): writing file permission denied")
        print("  Information:",excelfile)  
        return
    print("***Results saved in",excelfile,"@ sheet",sheetname)
    return  betas  
    
if __name__=='__main__':
    yearlist=['2013','2014','2015','2016','2017','2018','2019']
    excelfile='C:/temp/myexcel.xls'
    capm_beta_portfolio_yearly_excel(['AAPL','MSFT'],[2,1],'^GSPC',yearlist, excelfile,'p01')

#==============================================================================
def compare2_betas_yearly(stkcd1,stkcd2,mktidx,yearlist):
    """
    函数功能：分年度计算两只股票的CAPM模型贝塔系数，并绘图对比其风险特征的对冲性
    输入参数：
    stkcd1: 股票代码
    stkcd2: 股票代码
    mktidx: 指数代码
    yearlist: 年度列表，start日期要给第一个年份留出至少一年数据供回归使用
    输出数据：
    按年份绘图贝塔系数的变化
    无返回数据
    """
    #仅用于测试，完成后应注释掉
    #stkcd1='600028.SS'
    #stkcd2='600036.SS'
    #mktidx='000001.SS'
    #yearlist=['2013','2014','2015','2016','2017','2018','2019']

    
    #运行开始信息
    print("*** Comparing stock CAPM betas, please wait ...")
    
    #生成开始结束日期
    Y4=str(int(yearlist[0])-1)
    start=Y4+'-01-01'
    end=yearlist[-1]+'-12-31' 
        
    #读取股价并准备好收益率数据
    R1=prepare_capm_data(stkcd1,mktidx,start,end)
    if R1 is None:
        print("  #Error(compare2_betas_yearly): failed to prepare capm data")
        print("  Information:",stkcd1,mktidx,start,end)              
        return      

    R2=prepare_capm_data(stkcd2,mktidx,start,end)
    if R2 is None:
        print("  #Error(compare2_betas_yearly): failed to prepare capm data")
        print("  Information:",stkcd2,mktidx,start,end)              
        return 

    #用于保存beta(CAPM)
    import pandas as pd
    betas1=pd.DataFrame(columns=('Year','Beta','alpha','R-sqr','p-value'))
    #分年度OLS回归
    from scipy import stats
    for year in yearlist:
        r=R1[R1['Year']==year]
        if len(r)==0: continue
        try:
            output=stats.linregress(r['Close_x'],r['Close_y'])
        except:
            print("  #Error(compare2_betas_yearly): no data for regression")
            print("  Information:",stkcd1,year,r,R1)
            continue         
        (beta,alpha,r_value,p_value,std_err)=output
        
        row=pd.Series({'Year':year,'Beta':beta,'alpha':alpha, \
                       'R-sqr':r_value**2,'p-value':p_value})
        betas1=betas1.append(row,ignore_index=True)          
    betas1.set_index('Year',inplace=True)

    betas2=pd.DataFrame(columns=('Year','Beta','alpha','R-sqr','p-value'))
    for year in yearlist:
        #print(year,' ',end='')
        r=R2[R2['Year']==year]
        if len(r)==0: continue
        try:
            output=stats.linregress(r['Close_x'],r['Close_y'])
        except:
            print("  #Error(compare2_betas_yearly): no data for regression")
            print("  Information:",stkcd2,year,r,R2)
            continue         
        (beta,alpha,r_value,p_value,std_err)=output        
        row=pd.Series({'Year':year,'Beta':beta,'alpha':alpha, \
                       'R-sqr':r_value**2,'p-value':p_value})
        betas2=betas2.append(row,ignore_index=True)          
    betas2.set_index('Year',inplace=True)
    
    #绘图：年度贝塔系数变化
    plt.plot(betas1['Beta'],label=codetranslate(stkcd1),c='red',marker='o',lw=1)
    plt.plot(betas2['Beta'],label=codetranslate(stkcd2),c='blue',marker='D',lw=2,linestyle='--')
    plt.axhline(y=1.0,color='b',linestyle=':',c='green',label='市场平均风险')  
    plt.ylabel("贝塔系数",fontweight='bold')
    plt.xticks(rotation=45,fontweight='bold')
    trtitle="比较证券之间的贝塔系数"+"\n"+codetranslate(stkcd1)+" vs "+codetranslate(stkcd2)
    plt.title(trtitle,fontweight='bold')
    plt.legend(loc='best')
    
    import datetime; today = datetime.date.today()
    plt.xlabel("数据来源：新浪财经/雅虎财经，基于"+codetranslate(mktidx)+'，'+str(today))
    plt.show()
    
    return  betas1,betas2  
    
if __name__=='__main__':
    yearlist=['2013','2014','2015','2016','2017','2018','2019']
    compare2_betas_yearly('AMZN','WMT','^GSPC',yearlist)    
    compare2_betas_yearly('601857.SS','600036.SS','000001.SS',yearlist)

#==============================================================================
#==============================================================================
#==============================================================================